/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 



#ifndef ABSTRACTLISTMODEL_H
#define ABSTRACTLISTMODEL_H

#include "Object.h"
#include "ListModel.h"
#include "ModelValidationListener.h"
#include "ListModelListener.h"
#include "ListItem.h"
#include <vector>
#include "Persistable.h"

namespace VCF{

class Stream;


/**
*The AbstractListModel is basic implementation of ListModel. Deriving classes can use it as 
*starting point in their own implementation. See ListModel for more information on the methods
*implemented here.
*
*@version 1.0
*@author Jim Crafton
*/

#define ABSTRACTLISTMODEL_CLASSID		"ED88C0A7-26AB-11d4-B539-00C04F0196DA"

class APPKIT_API AbstractListModel : public ListModel, public Persistable {
public:
	BEGIN_ABSTRACT_CLASSINFO(AbstractListModel, "VCF::AbstractListModel", "VCF::ListModel", ABSTRACTLISTMODEL_CLASSID)
	END_CLASSINFO(AbstractListModel)

	AbstractListModel();

	virtual ~AbstractListModel();

	/**
	*This macro declares a std::vector<> of ListModelListener listeners
	*/
	LISTENER_LIST(ListModelListener);

	/**
	*This macro declares a std::vector<> of ModelValidationListener listeners
	*/
	LISTENER_LIST(ModelValidationListener);

    /**
	*This macro creates a method for adding a listener to the ListModelListener's ModelValidationListener events
	*/
	ADD_LISTENER(ModelValidationListener);

	/**
	*This macro creates a method for removing a listener to the ListModelListener's ModelValidationListener events
	*/
	REMOVE_LISTENER(ModelValidationListener);

    /**
     * validate the model. 
     * The implementation for this can vary widely, or even be nonexistant for model's that do not require validation.
     * The basic idea is to call all the listeners in the list , passing in a local variable to the
     * onModelValidate() methods of the listener's. The variable is initialized to true, and if it is
     * still true at the end of the listener iterations, then it is safe to apply the changes to the
     * model, other wise the changes are removed. 
     */
    virtual void validate();

    /**
    * clears out the model's data 
    */
    virtual void empty();
	
	/**
	*This macro creates a method for adding a listener to the ListModel's ListModelListener events
	*/
	ADD_LISTENER(ListModelListener);

	/**
	*This macro creates a method for removing a listener to the ListModel's ListModelListener events
	*/
	REMOVE_LISTENER(ListModelListener);   

	NOTIFIER(ListModelListener, onListModelContentsChanged, ListModelEvent);

	NOTIFIER(ListModelListener, onItemAdded, ListModelEvent);

	NOTIFIER(ListModelListener, onItemDeleted, ListModelEvent);

    virtual void deleteItem(ListItem * item);

    virtual void deleteItem(const unsigned long & index);

    virtual void insertItem(const unsigned long & index, ListItem * item);

    virtual void addItem(ListItem * item);

	virtual ListItem* getItemFromIndex( const unsigned long& index );

	virtual Enumerator<ListItem*>* getItems();

	virtual unsigned long getCount();

	/**
	*Write the object to the specified output stream
	*/
    virtual void saveToStream( OutputStream * stream );
	
	/**
	**Read the object from the specified input stream
	*/
    virtual void loadFromStream( InputStream * stream );	
private:    

    /**
    * @supplierCardinality 0..* 
   */
	std::vector<ListItem*> m_listItems;
	EnumeratorContainer<std::vector<ListItem*>, ListItem*> m_listContainer;
};

};

#endif //ABSTRACTLISTMODEL_H