/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// AbstractTextModel.h

#ifndef _ABSTRACTTEXTMODEL_H__
#define _ABSTRACTTEXTMODEL_H__


#include "TextModel.h"
#include "ValidationEvent.h"
#include "ModelValidationListener.h"
#include <vector>

namespace VCF
{

#define ABSTRACTTEXTMODEL_CLASSID	"8B2CDC33-3CAD-11d4-B553-00C04F0196DA"

/**
*
*/
class APPKIT_API AbstractTextModel : public TextModel  
{
public:
	BEGIN_ABSTRACT_CLASSINFO(AbstractTextModel, "VCF::AbstractTextModel", "VCF::TextModel", ABSTRACTTEXTMODEL_CLASSID)	
	END_CLASSINFO(AbstractTextModel)

	AbstractTextModel();

	virtual ~AbstractTextModel();

	/**
	*This macro declares a std::vector<> of TextModelListener listeners
	*/
	LISTENER_LIST(TextModelListener);

	/**
	*This macro declares a std::vector<> of ModelValidationListener listeners
	*/
	LISTENER_LIST(ModelValidationListener);

	/**
	*This macro creates a method for adding a listener to the TextModel's Model events
	*/
	ADD_LISTENER(TextModelListener);

	/**
	*This macro creates a method for removing a listener to the TextModel's Model events
	*/
	REMOVE_LISTENER(TextModelListener);
	
	/**
	*This macro creates a method for adding a listener to the TextModel's ModelValidationListener events
	*/
	ADD_LISTENER(ModelValidationListener);

	/**
	*This macro creates a method for removing a listener to the TextModel's ModelValidationListener events
	*/
	REMOVE_LISTENER(ModelValidationListener);


	NOTIFIER(TextModelListener, onTextChanged, TextEvent);    

	NOTIFIER(ModelValidationListener, onModelValidate, ValidationEvent );    

	/**
     * validate the model. 
     * The implementation for this can vary widely, or even be nonexistant for model's that do not require validation.
     * The basic idea is to call all the listeners in the list , passing in a local variable to the
     * onModelValidate() methods of the listener's. The variable is initialized to true, and if it is
     * still true at the end of the listener iterations, then it is safe to apply the changes to the
     * model, other wise the changes are removed. 
     */
    virtual void validate();

    /**
     * clears out the model's data 
     */
    virtual void empty();

	/**
	*sets the contents of the text model, completely changes what was previously 
	*in the model
	*/
    virtual void setText( const String& text );

	/**
	*inserts text into the model at the given index
	*@param long the starting point to begin inserting the text. This number represents
	*a zero based index.
	*@param String the text to insert
	*/
    virtual void insertText( const unsigned long& index, const String& text );

	/**
	*deletes text from the model, starting at index, and continuing for count characters, 
	*or until the text data is empty.
	*@param long the starting point. The index is zero based.
	*@param long the number of characters to delete
	*/
    virtual void deleteText( const unsigned long& index, const unsigned long& count );

	/**
	*adds text to end of the current text data
	*/
    virtual void appendText( const String& text );

	/**
	*returns all of the TextModel's text in a string.
	*/
	virtual String getText();

	/**
	*returns the size of the TextModel
	*/
	virtual unsigned long getSize();

protected:
	String m_text;
};

};


#endif 
