/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 



#ifndef ABSTRACTTREEMODEL_H
#define ABSTRACTTREEMODEL_H


#include "Object.h"
#include "TreeModel.h"
#include "ModelValidationListener.h"
#include "TreeModelListener.h"
#include "TreeItem.h"
#include "ItemListener.h"
#include <vector>

namespace VCF{

#define ABSTRACTTREEMODEL_CLASSID		"3126B227-2819-11d4-B53A-00C04F0196DA"


/**
*AbstractTreeModel is a basic implementation of Treemodel. It used for
*deriving classes to have a starting point for implementing TreeModel. 
*For more information on the methods implemented here please see TreeModel.
*
*@version 1.0
*@author Jim Crafton
*/
class AbstractTreeModel : public TreeModel, public ItemListener {
public:

	BEGIN_CLASSINFO(AbstractTreeModel, "VCF::AbstractTreeModel", "VCF::TreeModel", ABSTRACTTREEMODEL_CLASSID)	
	END_CLASSINFO(AbstractTreeModel)

    AbstractTreeModel();

	virtual ~AbstractTreeModel();    

	void init();
	/**
	*This macro declares a std::vector<> of TreeModelListener listeners
	*/
	LISTENER_LIST(TreeModelListener);

	/**
	*This macro declares a std::vector<> of ModelValidationListener listeners
	*/
	LISTENER_LIST(ModelValidationListener);

    /**
	*This macro creates a method for adding a listener to the TreeModelListener's ModelValidationListener events
	*/
	ADD_LISTENER(ModelValidationListener);

	/**
	*This macro creates a method for removing a listener to the TreeModelListener's ModelValidationListener events
	*/
	REMOVE_LISTENER(ModelValidationListener);

	/**
	*This macro creates a method for adding a listener to the TreeModelListener's TreeModelListener events
	*/
	ADD_LISTENER(TreeModelListener);

	/**
	*This macro creates a method for removing a listener to the TreeModelListener's TreeModelListener events
	*/
	REMOVE_LISTENER(TreeModelListener);

	NOTIFIER(TreeModelListener, onRootNodeChanged, TreeModelEvent);

	NOTIFIER(TreeModelListener, onNodeAdded, TreeModelEvent);
    /**
     * validate the model. 
     * The implementation for this can vary widely, or even be nonexistant for model's that do not 
	 *require validation.
     * The basic idea is to call all the listeners in the list , passing in a local variable to the
     * onModelValidate() methods of the listener's. The variable is initialized to true, and if it is
     * still true at the end of the listener iterations, then it is safe to apply the changes to the
     * model, other wise the changes are removed. 
     */
    virtual void validate();

    /**
     * clears out the model's data 
     */
    virtual void empty();

    virtual TreeItem* getRoot();    
	
	virtual void setRoot( TreeItem* root );

    virtual void insertNodeItem(TreeItem * node, TreeItem * nodeToInsertAfter);

    virtual void deleteNodeItem(TreeItem * nodeToDelete);

    virtual void addNodeItem( TreeItem * node, TreeItem * nodeParent=NULL );

    virtual void sort();
	
	virtual void onItemPaint( ItemEvent* event );

    virtual void onItemChanged( ItemEvent* event );

    virtual void onItemSelected( ItemEvent* event );

	virtual void onItemAdded( ItemEvent* event );
	
	virtual void onItemDeleted( ItemEvent* event );
protected:
	
private:    
	TreeItem* m_rootNodeItem;
	
};

};
#endif //ABSTRACTTREEMODEL_H