/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 


//Application.h



#ifndef APPLICATION_H
#define APPLICATION_H


#include "Object.h"
#include <vector>
#include <map>

namespace VCF
{

class Window;

class ApplicationPeer;

class ResourceBundle;

class WindowEvent;

class WindowHandler;

class Frame;

class Library;

/**
 *Represents the main process for a application. 
 *Each application contains a main window, pointed to by  m_mainWindow. 
 *Application are started by some entry point of the executing process.
 *On Win32 this is represented as a WinMain() function. On the other Windowing
 *systems this is more typcally a main() function. Once the entry point has been
 *called, the static function Application::appMain() is called starting the application to initialize it self.
 *
 *@version 1.0
 *@author Jim Crafton
 */

class APPKIT_API Application : public Object {
public:
	Application();

	virtual ~Application();

	/**
	*The entry point into the application object starting up. 
	*/
	static void appMain( const std::vector<String>& appCmdLine );
	
	static void appMain( int argc, char** argv );

	static Application* getRunningInstance();

	/**
	*sets the command line arguments for the application. called by Application::appMain() when 
	*the application is first starting up
	*/
	void setCommandLineArgs( const std::vector<String>& appCmdLine );

	/**
	*starts the application windows message pump running
	*/
	void run();
	
	/**
	*This happens after the Application object is constructed, after the initialize() method,
	*and just before the app's run() method is called
	*successful. if false, it calls terminate() and then kills the app process.
	*this method also builds the accellerator table used for keyboard shortcuts
	*@return bool this tells whether or not the initialization of the application was 
	*/
	virtual bool initRunningApplication();
	
	/**
	*terminates the running application
	*/
	virtual void terminateRunningApplication();


	/**
	*retreives the system specific Peer for this application
	*@return ApplicationPeer - the application Peer
	*/
	ApplicationPeer* getPeer();

	/**
	*gets the main window of the application. All applications have a main window of some
	*sort. This can a be a tool type window, a dialog, or just a regular window. When this
	*window is closed, the application begins it's shutdown priocess
	*@return Window* the main window
	*/
	Window* getMainWindow();

	/**
	*sets the main window for the application
	*/
	void setMainWindow( Window* mainWindow );

	/**
	*event handler for the main window closing down.
	*/
	void onMainWindowClose( WindowEvent* event );

	ResourceBundle* getResourceBundle();

	/**
	*returns the full path for the application executable
	*/
	String getFileName();


	/**
	*Creates a Frame from a frame derived class name
	*@param String the name of the frame derived class name
	*this is also the name of the VFF (Visual Form File) resource
	*which is loaded. If this can't be found or the class name
	*doesn't exist in the ClassRegistry then the return is NULL.
	*@return Frame a pointer to new instance of the type specified
	*in the frameClassName argument. NULL if the form resource doesn't exist
	*or the class doesn't exist. Otherwise it represent a full loaded 
	*instance, using the data from the resource VFF.
	*/
	Frame* createFrame( const String& frameClassName );

	/**
	*Loads an already existing Frame from a frame derived class name	
	*@param Frame a pointer to new instance of the type specified
	*in the frameClassName argument. 
	*/
	void loadFrame( Frame** frame );
	/**
	*load a VPL, calls the initPackage() function, and adds the library
	*to it's list. The library will be unloaded when the app quits.
	*/
	void loadVPL( const String& vplFileName );

	String getCurrentDirectory();

	void setCurrentDirectory( const String& currentDirectory );
protected:
	static Application* m_appInstance;
	std::vector<String> m_cmdLine;
	std::map<String,Library*> m_VPLMap;
private:    
    Window * m_mainWindow;
	ApplicationPeer* m_applicationPeer;
	WindowHandler* m_mainWinHandler;
};

};


#endif //APPLICATION_H__