/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// ClassRegistry.h

#ifndef _CLASSREGISTRY_H__
#define _CLASSREGISTRY_H__

#pragma warning (disable : 4786)
#include <iostream>

#include "Object.h"
#include <map>
#include "Class.h"
#include "Method.h"
#include "Property.h"
#include "CommonDefines.h"
#include "ClassNotFound.h"
#include "Enumerator.h"
#include "StringUtils.h"

namespace VCF
{

/**
*ClassRegistry contains all Class's in the Framework Runtime.
*The ClassRegistry is used by the Framework Runtime to register Classes 
*and properties for Classes. The ClassRegistry is also used to dynamically 
*create new instances of a class from a class name.
*
*@author Jim Crafton
*@version 1.0
*/

class FRAMEWORK_API ClassRegistry : public Object  
{
public:
	ClassRegistry();

	virtual ~ClassRegistry();	

	/**
	*a static function that returns the global instance of the ClassRegistry.
	*/
	static ClassRegistry* getClassRegistry();
	
	/**
	*a map of all the classes in the ClassRegistry. keyed by the class name 
	*/
	std::map<String,Class*> m_classMap;	

	/**
	*returns a Class object from a string. If no matching 
	*Class exists the function returns NULL.
	*In terms of efficiency, this function is faster than 
	*getClass( Object*)
	*@param String the name of the class to find
	*@return Class the class associated with the className
	*/
	Class* getClass( const String& className );	

	/**
	*returns a Class object for the object passed in.
	*@param Object the object instance to get a class for
	*@return Class the class associated with the object
	*/
	Class* getClass( Object* object );

	/**
	*creates a new instance of an object based on the class name.
	*@param String the class name of the object to create
	*@param Object a pointer to the newly created object
	*/
	void createNewInstance( const String& className, Object** newInstance );	

	/**
	*adds a new Class object to the ClassRegistry
	*@param String the name of the class
	*@param Class the Class object to register
	*/
	void addClass( const String& className, Class* classToRegister );

	Enumerator<Class*>* getClasses();

	void dump();

	/**
	*the global ClassRegistry instance.
	*/
	static ClassRegistry m_registryInstance;

private:
	EnumeratorMapContainer<std::map<String,Class*>,Class*> m_classContainer;
	
};


/**
*this function registers a property with a class, where the PROPERTY_TYPE is defined as some 
*type like int, double, bool, Object*, etc.
*
*@param PROPERTY_TYPE initialValue is used partly for an initial value for the 
*property, but mainly to make sure thr template function is created properly.
*@param String className - the name of the class to associatent the property with
*@param String propertyName - the name of the property
*@param TypedProperty<PROPERTY_TYPE>::GetFunction - the property's get function, allows for 
*retreiving values from the property
*@param TypedProperty<PROPERTY_TYPE>::SetFunction - the property's set function, allows for 
*setting values on the property
*@param PropertyDescriptorType the property type
*/
template <class PROPERTY_TYPE>  void registerProperty( const String& className,
                                                 const String& propertyName,
                                                 TypedProperty<PROPERTY_TYPE>::GetFunction propertyGetFunction,
                                                 TypedProperty<PROPERTY_TYPE>::SetFunction propertySetFunction,
                                                 const PropertyDescriptorType& propertyFieldDescriptor ){
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasProperty( propertyName ) ){
			TypedProperty<PROPERTY_TYPE>* newProperty = 
							new TypedProperty<PROPERTY_TYPE>( propertyGetFunction, 
							                             propertySetFunction, 
														 propertyFieldDescriptor );
			newProperty->setName( propertyName );
			clazz->addProperty( newProperty );
		}
	}
}

/**
*this function registers a READ-ONLY property with a class, where the PROPERTY_TYPE is defined as some 
*basic type like int, double, bool, etc.
*
*class PROPERTY_TYPE is the template type to base the function on
*@param String className - the name of the class to associate the property with
*@param String propertyName - the name of the property
*@param TypedProperty<PROPERTY_TYPE>::GetFunction - the property's get function, allows for 
*retreiving values from the property
*@param PropertyDescriptorType the property type
*/
template <class PROPERTY_TYPE>  void registerReadOnlyProperty( 
												 const String& className, 
												 const String& propertyName,
												 TypedProperty<PROPERTY_TYPE>::GetFunction propertyGetFunction,												 
												 const PropertyDescriptorType& propertyFieldDescriptor ){
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasProperty( propertyName ) ){
			TypedProperty<PROPERTY_TYPE>* newProperty = 
							new TypedProperty<PROPERTY_TYPE>( propertyGetFunction, 
														 propertyFieldDescriptor );
			newProperty->setName( propertyName );
			clazz->addProperty( newProperty );
		}
	}
}

/**
*this function registers a READ-ONLY property with a class, where the PROPERTY_TYPE is defined as some 
*Oject* derived type
*
*class PROPERTY_TYPE is the template type to base the function on
*@param String className - the name of the class to associate the property with
*@param String propertyName - the name of the property
*@param TypedObjectProperty<PROPERTY_TYPE>::GetFunction - the property's get function, allows for 
*retreiving values from the property
*/
template <class PROPERTY_TYPE>  void registerReadOnlyProperty( const String& className, 
												         const String& propertyName,
												         TypedObjectProperty<PROPERTY_TYPE>::GetFunction propertyGetFunction ){
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasProperty( propertyName ) ){
			TypedObjectProperty<PROPERTY_TYPE>* newProperty = 
							new TypedObjectProperty<PROPERTY_TYPE>( propertyGetFunction );
			newProperty->setName( propertyName );
			clazz->addProperty( newProperty );
		}
	}
}

/**
*this function registers a READ-ONLY property with a class, where the PROPERTY_TYPE is defined as some 
*enum type
*
*class PROPERTY_TYPE is the template type to base the function on
*@param String className - the name of the class to associate the property with
*@param String propertyName - the name of the property
*@param TypedProperty<PROPERTY_TYPE>::GetFunction - the property's get function, allows for 
*retreiving values from the enum property
*@param PROPERTY_TYPE the lower bound of enum
*@param PROPERTY_TYPE the upper bound of enum
*/
template <class PROPERTY_TYPE>  void registerReadOnlyProperty( const String& className, const String& propertyName,
												         TypedEnumProperty<PROPERTY_TYPE>::GetFunction propertyGetFunction,
												         PROPERTY_TYPE lower, PROPERTY_TYPE upper ){
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasProperty( propertyName ) ){
			TypedEnumProperty<PROPERTY_TYPE>* newProperty = 
							new TypedEnumProperty<PROPERTY_TYPE>( propertyGetFunction, lower, upper );
			newProperty->setName( propertyName );
			clazz->addProperty( newProperty );
		}
	}
};

/**
*this function registers a READ-ONLY property with a class, where the PROPERTY_TYPE is defined as some 
*enum type. In addition, this also associates a human readable string with each enum value. These
*value's can then be represented more descriptively in an IDE than just their integer equivalent. 
*
*class PROPERTY_TYPE is the template type to base the function on
*@param String className - the name of the class to associate the property with
*@param String propertyName - the name of the property
*@param TypedProperty<PROPERTY_TYPE>::GetFunction - the property's get function, allows for 
*retreiving values from the enum property
*@param PROPERTY_TYPE the lower bound of enum
*@param PROPERTY_TYPE the upper bound of enum
*@param unsigned long the number of string names enumNames points to
*@param String a pointer to an array of Strings that holds a human 
*readable value for each enum type.
*/
template <class PROPERTY_TYPE>  void registerReadOnlyProperty( const String& className, const String& propertyName,
												         TypedEnumProperty<PROPERTY_TYPE>::GetFunction propertyGetFunction,
												         PROPERTY_TYPE lower, PROPERTY_TYPE upper,
												         const unsigned long& enumNameCount, 
												         String* enumNames ){
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasProperty( propertyName ) ){
			TypedEnumProperty<PROPERTY_TYPE>* newProperty = 
							new TypedEnumProperty<PROPERTY_TYPE>( propertyGetFunction, 
															 NULL,			
															 lower, upper,
															 enumNameCount,
															 enumNames );
			newProperty->setName( propertyName );
			clazz->addProperty( newProperty );
		}
	}
};

/**
*this function registers a property with a class, where the PROPERTY_TYPE is defined as some 
*Object* derived type
*
*class PROPERTY_TYPE is the template type to base the function on
*@param String className - the name of the class to associate the property with
*@param String propertyName - the name of the property
*@param TypedObjectProperty<PROPERTY_TYPE>::GetFunction - the property's get function, allows for 
*retreiving values from the property
*@param TypedObjectProperty<PROPERTY_TYPE>::SetFunction - the property's set function, allows for 
*setting values on the property
*/
template <class PROPERTY_TYPE>  void registerProperty( const String& className, const String& propertyName,
												TypedObjectProperty<PROPERTY_TYPE>::GetFunction propertyGetFunction,
												TypedObjectProperty<PROPERTY_TYPE>::SetFunction propertySetFunction ){
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasProperty( propertyName ) ){
			TypedObjectProperty<PROPERTY_TYPE>* newProperty = 
							new TypedObjectProperty<PROPERTY_TYPE>( propertyGetFunction, 
							                             propertySetFunction );
			newProperty->setName( propertyName );
			clazz->addProperty( newProperty );
		}
	}
};

/**
*this function registers a property with a class, where the PROPERTY_TYPE is defined as some 
*enum type
*
*class PROPERTY_TYPE is the template type to base the function on
*@param String className - the name of the class to associate the property with
*@param String propertyName - the name of the property
*@param TypedEnumProperty<PROPERTY_TYPE>::GetFunction - the property's get function, allows for 
*retreiving values from the property
*@param TypedEnumProperty<PROPERTY_TYPE>::SetFunction - the property's set function, allows for 
*setting values on the property
*@param PROPERTY_TYPE the lower bound of enum
*@param PROPERTY_TYPE the upper bound of enum
*/
template <class PROPERTY_TYPE>  void registerProperty( const String& className, const String& propertyName,
												 TypedEnumProperty<PROPERTY_TYPE>::GetFunction propertyGetFunction,
												 TypedEnumProperty<PROPERTY_TYPE>::SetFunction propertySetFunction,
												 PROPERTY_TYPE lower, PROPERTY_TYPE upper ){
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasProperty( propertyName ) ){
			TypedEnumProperty<PROPERTY_TYPE>* newProperty = 
							new TypedEnumProperty<PROPERTY_TYPE>( propertyGetFunction, 
							                                 propertySetFunction, lower, upper );
			newProperty->setName( propertyName );
			clazz->addProperty( newProperty );
		}
	}
};

/**
*this function registers a property with a class, where the PROPERTY_TYPE is defined as some 
*enum type. In addition, this also associates a human readable string with each enum value. These
*value's can then be represented more descriptively in an IDE than just their integer equivalent. 
*
*class PROPERTY_TYPE is the template type to base the function on
*@param String className - the name of the class to associate the property with
*@param String propertyName - the name of the property
*@param TypedProperty<PROPERTY_TYPE>::GetFunction - the property's get function, allows for 
*retreiving values from the enum property
*@param TypedEnumProperty<PROPERTY_TYPE>::SetFunction - the property's set function, allows for 
*setting values on the property
*@param PROPERTY_TYPE the lower bound of enum
*@param PROPERTY_TYPE the upper bound of enum
*@param unsigned long the number of string names enumNames points to
*@param String a pointer to an array of Strings that holds a human 
*readable value for each enum type.
*/
template <class PROPERTY_TYPE>  void registerProperty( const String& className, const String& propertyName,
												 TypedEnumProperty<PROPERTY_TYPE>::GetFunction propertyGetFunction,
												 TypedEnumProperty<PROPERTY_TYPE>::SetFunction propertySetFunction,
												 PROPERTY_TYPE lower, PROPERTY_TYPE upper,
												 const unsigned long& enumNameCount, 
												 String* enumNames ){
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasProperty( propertyName ) ){
			TypedEnumProperty<PROPERTY_TYPE>* newProperty = 
							new TypedEnumProperty<PROPERTY_TYPE>( propertyGetFunction, 
							                                 propertySetFunction, 
															 lower, upper,
															 enumNameCount,
															 enumNames );
			newProperty->setName( propertyName );
			clazz->addProperty( newProperty );
		}
	}
};


template <class PROPERTY_TYPE>  void registerCollectionProperty( const String& className, const String& propertyName,
												           TypedCollectionProperty<PROPERTY_TYPE>::GetFunction propertyGetFunction,
												           const PropertyDescriptorType& propertyFieldDescriptor ){
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasProperty( propertyName ) ){
			TypedCollectionProperty<PROPERTY_TYPE>* newProperty = 
							new TypedCollectionProperty<PROPERTY_TYPE>( propertyGetFunction, 
							                                       propertyFieldDescriptor );
			newProperty->setName( propertyName );
			clazz->addProperty( newProperty );
		}
	}
};

/**
*this function registers a property with a class, where the PROPERTY_TYPE is defined as some 
*Object* derived type in a collection. In registering the collection, you must provide ways 
*to enumerate, add, insert, and delete from the collection. 
*
*class PROPERTY_TYPE is the template type to base the function on
*@param String className - the name of the class to associate the property with
*@param String propertyName - the name of the property
*@param TypedCollectionProperty<PROPERTY_TYPE>::GetFunction - the property's get function, allows for 
*retreiving values from the enum property
*@param TypedCollectionProperty<PROPERTY_TYPE>::AddFunction the collection's add function. Takes an
*item of PROPERTY_TYPE type
*@param TypedCollectionProperty<PROPERTY_TYPE>::InsertFunction the collection's insert function. Takes an
*item of PROPERTY_TYPE type and an index
*@param TypedCollectionProperty<PROPERTY_TYPE>::DeleteFunction1 the collection's delete function. Takes an
*item of PROPERTY_TYPE type
*@param TypedCollectionProperty<PROPERTY_TYPE>::DeleteFunction2 the collection's delete function. Takes 
*an index
*/
template <class PROPERTY_TYPE>  void registerCollectionProperty( const String& className, const String& propertyName,
												           TypedObjectCollectionProperty<PROPERTY_TYPE>::GetFunction propertyGetFunction,
														   TypedObjectCollectionProperty<PROPERTY_TYPE>::AddFunction propertyAddFunction, 
														   TypedObjectCollectionProperty<PROPERTY_TYPE>::InsertFunction propertyInsertFunction,
														   TypedObjectCollectionProperty<PROPERTY_TYPE>::DeleteFunction1 propertyDeleteFunction1,
														   TypedObjectCollectionProperty<PROPERTY_TYPE>::DeleteFunction2 propertyDeleteFunction2 ){
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasProperty( propertyName ) ){
			TypedObjectCollectionProperty<PROPERTY_TYPE>* newProperty = 
							new TypedObjectCollectionProperty<PROPERTY_TYPE>( propertyGetFunction,
							                                             propertyAddFunction,
																		 propertyInsertFunction, 
																		 propertyDeleteFunction1,
																		 propertyDeleteFunction2 );
			newProperty->setName( propertyName );
			clazz->addProperty( newProperty );
		}
	}
};

/**
*registers a class in the global ClassRegistry instance.
*@param T - a dummy param to make the &*$#$% C++ compiler happy and ensure the 
*template function is created correctly. It would be nice to remove this because 
*it not used at all in the function
*@param String the name of the class to register
*@param String the name of the class's super class
*@param String the classID of the class to register. This MUST be 
*a string that represents unique ID as returned by a UUID function/algorithm
*
*@return bool true if the class has been registered, otherwise false
*/
template <class T> bool registerClass( T* fakeParam, const String& className, 
									   const String& superClassName, const String& classID ){
	
	bool result = false;
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	
	//StringUtils::trace( "registerClass for " + String(typeid(T).name()) + "\n" );
	
	Class* clazz = classRegistry->getClass( className );

	if ( NULL == clazz ){		
		//StringUtils::trace( "registering new class " + className + " with ClassRegistry\n" );

		TypedClass<T>* objClass = new TypedClass<T>( className, classID, superClassName );

		classRegistry->addClass( className, objClass );

		result = true;
	}
	return result;
};

template <class T>  bool registerAbstractClass( T* fakeParam, const String& className, 
									           const String& superClassName, const String& classID ){
	
	bool result = false;
	//StringUtils::trace( "registerAbstractClass for " + String(typeid(T).name()) + "\n" );
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();

	Class* clazz = classRegistry->getClass( className );

	if ( NULL == clazz ){		
		//StringUtils::trace( "registering new abstract class " + className + " with ClassRegistry\n" );
		TypedAbstractClass<T>* objClass = new TypedAbstractClass<T>( className, classID, superClassName );

		classRegistry->addClass( className, objClass );

		result = true;
	}
	return result;
};

/**
*registers a new event witha class. Calls the Class::addEvent()
*if a class is found for the given clasName. Internally the class will
*create a new EventSet class if the event set did not previously exist 
*and the add the new event.
*MSVC note:
*the inline keyword was added to make the linker happy and stop it bitching
*about linker warning 4006. 
*/
 inline FRAMEWORK_API bool registerEvent( const String& className, const String& eventClassName, const String& eventMethodName, 
					  const String& listenerClassName, const String& adapterClassName ){
	bool result = false;
	
	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();

	Class* clazz = classRegistry->getClass( className );	

	if ( NULL != clazz ){
		clazz->addEvent( eventClassName, eventMethodName, listenerClassName, adapterClassName );
		result = true;
	}
	return result;
}


/**
*registers a new method with a Class
*@param T - a dummy param to make the &*$#$% C++ compiler happy and ensure the 
*template function is created correctly. It would be nice to remove this because 
*it not used at all in the function
*@param String the name of the class to register
*@param String the name of the class's method to register
*@param METHOD_TYPE the method pointer of the class
*@param String the arg types of the method
*@param bool doe the method have a return value
*@param ulong32 the number of arguments of the method
*
*@return bool true if the method has been registered, otherwise false
*/
template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerMethod0Return( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
													    METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );

	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			Method* newMethod = new TypedMethodArg0Return<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );

			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
			
		}
	}

};


template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerMethod1Return( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
													    METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );

	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			Method* newMethod = new TypedMethodArg1Return<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );

			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
			
		}
	}

};

template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerMethod2Return( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
													    METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );

	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			Method* newMethod = new TypedMethodArg2Return<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );

			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
			
		}
	}

};

template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerMethod3Return( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
													    METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );

	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			Method* newMethod = new TypedMethodArg3Return<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );

			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
			
		}
	}

};

template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerMethod4Return( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
													    METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );

	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			Method* newMethod = new TypedMethodArg4Return<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );

			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
			
		}
	}

};

template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerMethod5Return( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
													    METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );

	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			Method* newMethod = new TypedMethodArg5Return<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );

			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
			
		}
	}

};


template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerMethod6Return( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
													    METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );

	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			Method* newMethod = new TypedMethodArg6Return<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );

			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
			
		}
	}

};

/**
*registers a new method with a Class
*@param T - a dummy param to make the &*$#$% C++ compiler happy and ensure the 
*template function is created correctly. It would be nice to remove this because 
*it not used at all in the function
*@param String the name of the class to register
*@param String the name of the class's method to register
*@param METHOD_TYPE the method pointer of the class
*@param String the arg types of the method
*@param bool doe the method have a return value
*@param ulong32 the number of arguments of the method
*
*@return bool true if the method has been registered, otherwise false
*/
 template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerVoidMethodArg0( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
														 METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			
			Method* newMethod = new TypedMethodArg0<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );
			
			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
		}
	}

};


template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerVoidMethodArg1( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
														 METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			
			Method* newMethod = new TypedMethodArg1<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );
			
			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
		}
	}

};

template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerVoidMethodArg2( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
														 METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			
			Method* newMethod = new TypedMethodArg2<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );
			
			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
		}
	}

};

template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerVoidMethodArg3( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
														 METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			
			Method* newMethod = new TypedMethodArg3<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );
			
			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
		}
	}

};

template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerVoidMethodArg4( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
														 METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			
			Method* newMethod = new TypedMethodArg4<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );
			
			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
		}
	}

};


template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerVoidMethodArg5( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
														 METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			
			Method* newMethod = new TypedMethodArg5<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );
			
			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
		}
	}

};


template<class SOURCE_TYPE, 
		 class METHOD_TYPE>  void registerVoidMethodArg6( SOURCE_TYPE* fakeParam, const String& className, const String& methodName, 
														 METHOD_TYPE methodPtr, const String& argTypes ){

	ClassRegistry* classRegistry = ClassRegistry::getClassRegistry();
	Class* clazz = classRegistry->getClass( className );
	if ( NULL != clazz ){
		if ( false == clazz->hasMethod( methodName ) ){
			
			Method* newMethod = new TypedMethodArg6<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes );
			
			if ( NULL != newMethod ){
				newMethod->setName( methodName );
				clazz->addMethod( newMethod );
			}
		}
	}

};


};


#endif 

