/**
*Visual Control Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Control Framework
*/ 
// Control.h

#ifndef _CONTROL_H__
#define _CONTROL_H__

#include "Object.h"
#include "Component.h"
#include "ControlEvent.h"
#include "ControlListener.h"
#include "MouseListener.h"
#include "KeyboardListener.h"
#include "KeyboardEvent.h"
#include "MouseEvent.h"

namespace VCF {



/**
 * The base class for all widgets in the Visual Control Framework.
 */

class Border;

class Font;

class ControlPeer;

class GraphicsContext;

class ControlGraphicsContext;

class PopupMenu;

#define CONTROL_CLASSID		"B91B1828-3639-4bcf-9882-342F16C90E21"

class APPKIT_API Control : public Component{

public:
	
	BEGIN_ABSTRACT_CLASSINFO(Control, "VCF::Control", "VCF::Component", CONTROL_CLASSID);
	PROPERTY( bool, "visible", Control::getVisible, Control::setVisible, PROP_BOOL );	
	PROPERTY( double, "width", Control::getWidth, Control::setWidth, PROP_DOUBLE );
	PROPERTY( double, "height", Control::getHeight, Control::setHeight, PROP_DOUBLE );
	PROPERTY( bool, "enabled", Control::isEnabled, Control::setEnabled, PROP_BOOL );
	PROPERTY( bool, "doubleBuffered", Control::isDoubleBuffered, Control::setDoubleBuffered, PROP_BOOL );
	PROPERTY( bool, "useParentFont", Control::useParentFont, Control::setUseParentFont, PROP_BOOL );
	PROPERTY( bool, "focused", Control::isFocused, Control::setFocus, PROP_BOOL );	
	PROPERTY( bool, "autoStartDragDrop", Control::getAutoStartDragDrop, Control::setAutoStartDragDrop, PROP_BOOL );
	OBJECT_PROPERTY( Border, "border", Control::getBorder, Control::setBorder );	
	OBJECT_PROPERTY( Color, "color", Control::getColor, Control::setColor );
	OBJECT_PROPERTY( Font, "font", Control::getFont, Control::setFont );
	OBJECT_PROPERTY( PopupMenu, "popupMenu", Control::getPopupMenu, Control::setPopupMenu );
	LABELED_ENUM_PROPERTY( AlignmentType, "alignment", Control::getAlignment, Control::setAlignment, 
						   ALIGN_NONE, ALIGN_CLIENT, 6, AlignmentTypeNames);	

	EVENT("VCF::ControlEvent", "onControlSized", "VCF::ControlListener", "VCF::ControlHandler" );
	EVENT("VCF::ControlEvent", "onControlPositioned", "VCF::ControlListener", "VCF::ControlHandler" );
	EVENT("VCF::ControlEvent", "onControlParentChanged", "VCF::ControlListener", "VCF::ControlHandler" );

	EVENT("VCF::MouseEvent", "onMouseDoubleClicked", "VCF::MouseListener", "VCF::MouseHandler" );
	EVENT("VCF::MouseEvent", "onMouseClicked", "VCF::MouseListener", "VCF::MouseHandler" );
	EVENT("VCF::MouseEvent", "onMouseMove", "VCF::MouseListener", "VCF::MouseHandler" );
	EVENT("VCF::MouseEvent", "onMouseUp", "VCF::MouseListener", "VCF::MouseHandler" );
	EVENT("VCF::MouseEvent", "onMouseDown", "VCF::MouseListener", "VCF::MouseHandler" );

	EVENT("VCF::KeyboardEvent", "onKeyPressed", "VCF::KeyboardListener", "VCF::KeyboardHandler" );
	EVENT("VCF::KeyboardEvent", "onKeyDown", "VCF::KeyboardListener", "VCF::KeyboardHandler" );
	EVENT("VCF::KeyboardEvent", "onKeyUp", "VCF::KeyboardListener", "VCF::KeyboardHandler" );

	END_CLASSINFO(Control);

	Control();	

	virtual ~Control();

	/**
	*This macro declares a std::vector<> of MouseListener listeners
	*/
	LISTENER_LIST(MouseListener);

	/**
	*This macro declares a std::vector<> of ControlListener listeners
	*/
	LISTENER_LIST(ControlListener);

	/**
	*This macro declares a std::vector<> of KeyboardListener listeners
	*/
	LISTENER_LIST(KeyboardListener);

	/**
	*This macro creates a method for adding a listener to the Control's Mouse events
	*/
	ADD_LISTENER(MouseListener);

	/**
	*This macro creates a method for removing a listener to the Control's Mouse events
	*/
	REMOVE_LISTENER(MouseListener);

	/**
	*This macro creates a method for adding a listener to the Control's events
	*/
	ADD_LISTENER(ControlListener);

	/**
	*This macro creates a method for removing a listener to the Control's events
	*/
	REMOVE_LISTENER(ControlListener);

	/**
	*This macro creates a method for adding a listener to the Control's Keyboard events
	*/
	ADD_LISTENER(KeyboardListener);

	/**
	*This macro creates a method for removing a listener to the Control's Keyboard events
	*/
	REMOVE_LISTENER(KeyboardListener);

	/**
	*macro adds event notifier methods. These are equivalent to the fireXXX methods
	*of java. The macro expands to code that automatically calls all the listeners and
	*fires off the appropriate listener method.
	*/
	/**
	*Mouse events
	*/
	NOTIFIER(MouseListener, onMouseDoubleClicked, MouseEvent);    

	NOTIFIER(MouseListener, onMouseClicked, MouseEvent);

	NOTIFIER(MouseListener, onMouseMove, MouseEvent);    
	
	NOTIFIER(MouseListener, onMouseUp, MouseEvent);

	NOTIFIER(MouseListener, onMouseDown, MouseEvent);

	/**
	*Control events
	*/
	NOTIFIER(ControlListener, onControlSized, ControlEvent);

	NOTIFIER(ControlListener, onControlPositioned, ControlEvent);	

	NOTIFIER(ControlListener, onControlParentChanged, ControlEvent);


	/**
	*Keyboard events
	*/
	NOTIFIER(KeyboardListener, onKeyPressed, KeyboardEvent);

	NOTIFIER(KeyboardListener, onKeyDown, KeyboardEvent);

	NOTIFIER(KeyboardListener, onKeyUp, KeyboardEvent);    

	Border* getBorder();

	void setBorder( Border* border );

    /**
     * returns the bounds in parent coordinates of the Control. The Control derived class must call it's Peer's getBounds() method 
     */
    virtual Rect* getBounds() throw( InvalidPeer );
	
	virtual Rect* getClientBounds( const bool& includeBorder = true ) throw( InvalidPeer );

    virtual double getLeft() throw( InvalidPeer );

    virtual double getWidth() throw( InvalidPeer );

    virtual double getTop() throw( InvalidPeer );

    virtual double getHeight() throw( InvalidPeer );

    virtual bool getVisible() throw( InvalidPeer );

    AlignmentType getAlignment();

    virtual void setBounds( Rect* rect ) throw( InvalidPeer );

    void setAlignment( const AlignmentType& alignment );

    virtual void setLeft( const double& left ) throw( InvalidPeer );

    virtual void setWidth( const double& width ) throw( InvalidPeer );

    virtual void paint( GraphicsContext * context )=0;

    virtual void setTop( const double& top ) throw( InvalidPeer );

    virtual void setHeight( const double& height ) throw( InvalidPeer );


    virtual void setVisible( const bool& visible ) throw( InvalidPeer );

    virtual void saveToStream( OutputStream * stream );

    virtual void loadFromStream( InputStream * stream );

    /**
     * This gets called by the ControlPeer.
     *  Once inside the event the Control determines the type, and behaves accordingly,
     * as well as notifying any appropriate listeners. 
     */
    void processMouseEvent( MouseEvent* mouseEvent );

    void processKeyboardEvent( KeyboardEvent* keyboardEvent );

    void processControlEvent( ControlEvent* ControlEvent );

    /**
     *if m_autoStartDragDrop is true then this method checks to see if the Controls dragging is with 
	 *the windowing system's tolernace for a drag drop operation to start. other wise it returns false. 
     *If  m_autoStartDragDrop is false then it returns false unless some derived class overides the functionality. 
     */
    virtual bool canBeginDragDrop();

    void setAutoStartDragDrop(const bool& canAutoStartDragDrop);

	bool getAutoStartDragDrop();

    virtual void beginDragDrop();
	
	virtual ControlPeer* getPeer();

	virtual void setParent( Control* parent ) throw( InvalidPeer );
	virtual Control* getParent() throw( InvalidPeer );

	bool isFocused();
	void setFocus( const bool& focused );

	bool isEnabled();
	void setEnabled( const bool& enabled );
	
	virtual void mouseDown( MouseEvent* event );

	virtual void mouseMove( MouseEvent* event );

	virtual void mouseUp( MouseEvent* event );	

	virtual void mouseClick(  MouseEvent* event );
	
	virtual void mouseDblClick(  MouseEvent* event );

	virtual void keyDown( KeyboardEvent* event );
	virtual void keyPressed( KeyboardEvent* event );
	virtual void keyUp( KeyboardEvent* event );

	void translateToParent( Point* point );
	void translateToLocal( Point* point );

	/**
	*identifies the control as a lightweight control. Lightweight control's
	*do not take up windowing system resources, instead, they rely on all events
	*and paint notification be sent to them via their parents. The default return for 
	*this is false, so only those controls specifically actign as light weight Controls
	*need to return true
	*/
	virtual bool isLighweight();

	/**
	*Returns the first parent of the Control that is a heavweight Control, i.e it's isLighweight()
	*method return false.
	*/
	virtual Control* getHeavyweightParent();

	virtual String toString();

	Color* getColor();

	void setColor( Color* color );

	virtual void beforeDestroy( ControlEvent* event );
	
	virtual void afterCreate( ComponentEvent* event );
	
	Font* getFont();

	void setFont( Font* font );

	bool useParentFont();

	void setUseParentFont( const bool& useParentFont );

	void repaint( Rect* repaintRect=NULL );

	/**
	*is this component double buffered
	*/
	bool isDoubleBuffered();

	void setDoubleBuffered( const bool& doubleBuffered );

	/**
	*this keeps the mouse events being sent to this control, even is the 
	*mouse leaves the physical bounds of the control
	*/
	void keepMouseEvents();

	/**
	*releases mouse events - goes back to normal event handling
	*/
	void releaseMouseEvents();

	/**
	*return a pointer to the graphics context of the control. 
	*/
	GraphicsContext* getContext();

	/**
	*returns the Control's PopupMenu, if any
	*/
	PopupMenu* getPopupMenu();

	void setPopupMenu( PopupMenu* popupMenu );

	static Control* getCapturedMouseControl();
	
	static void setCapturedMouseControl( Control* control );
protected:
		ControlPeer* m_Peer;
		ControlGraphicsContext* m_context;
private:
    Control * m_parent;
    AlignmentType m_aligment;
    Rect* m_bounds;
	Rect* m_clientBounds;
	Border* m_border;
	Color* m_color;
	Font* m_font;
	bool m_useParentFont;
	bool m_doubleBuffered;
	bool m_hasMouseCapture;	
	bool m_autoStartDragDrop;
	PopupMenu* m_popupMenu;
	static Control* capturedMouseControl;
};


};

#endif //_CONTROL_H__

