/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _METHOD_H__
#define _METHOD_H__

#include "VCF.h"
#include "Object.h"
#include "Property.h"


namespace VCF {


/**
*Base class for dynamic method invocation
*Methods are dynamically invokable and can support
*querying for argument types
*querying is done throught the getArgumentType() method
*and will only work if the m_argTypes string has bee set appropriately.
*This string consists of a series of characters each of which
*are interpreted as a special code for the primitive type.
*The code table is as follows:
<p>
<code>
	  code	|	primitive type
	----------------------------	
	  "i"   |		integer
	  "l"	|		long
	  "h"	|		short
	  "+l"	|		unsigned long
	  "f"	|		float
	  "c"	|		char
	  "d"	|		double
	  "o"	|		Object*
	  "b"	|		bool
	  "s"	|		String or String&
	  "e"	|		Enum* 

</code>
<p>

*/
class Method : public Object{

public:
	Method(){
		m_argCount = 0;	
		m_hasReturnValue = false;
	};

	virtual ~Method(){};

	/**
	*invoke the method on the source object
	*/
	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ) = 0;

	/**
	*invoke the method on the already bound source 
	*Assumes setSource() has already been called
	*/
	virtual VariantData* invokeOnSource( const uint32& argCount, 
										 VariantData** arguments ) = 0;
	String getName(){
		return m_name;
	}
	
	void setName( const String& name ){
		m_name = name;
	}

	ulong32 getArgCount(){
		return m_argCount;
	}

	bool hasReturnValue(){
		return m_hasReturnValue;
	}

	/**
	*returns the bound source of teh method - may return NULL if
	*no source has been set
	*/
	virtual Object* getSource() = 0;

	/**
	*sets the source of the object
	*/
	virtual void setSource( Object* source ) = 0;

	PropertyDescriptorType getArgumentType( const ulong32& argumentIndex ){
		PropertyDescriptorType result = PROP_UNDEFINED;
		VCFChar ch = m_argTypes[argumentIndex];

		switch ( ch ){
			case 'i' :{
				result = PROP_INT;
			}
			break;

			case 'l' :{
				result = PROP_LONG;
			}
			break;

			case 'h' :{
				result = PROP_SHORT;
			}
			break;

			case '+' :{
				VCFChar next = m_argTypes[argumentIndex+1];
				if ( 'l' == next ){
					result = PROP_ULONG;
				}
			}
			break;

			case 'f' :{
				result = PROP_FLOAT;
			}
			break;

			case 'c' :{
				result = PROP_CHAR;
			}
			break;

			case 'd' :{
				result = PROP_DOUBLE;
			}
			break;

			case 'o' :{
				result = PROP_OBJECT;
			}
			break;

			case 'b' :{
				result = PROP_BOOL;
			}
			break;

			case 'e' :{
				result = PROP_ENUM;
			}
			break;

			case 's' :{
				result = PROP_STRING;
			}
			break;
		}
		return result;
	}
protected:
	String m_argTypes;
	ulong32 m_argCount;
	bool m_hasReturnValue;
private:
	String m_name;
	
};



/**
*Base template class for methodsthat do NOT return values
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethod : public Method {
public:
	TypedMethod( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ){
		m_methodPtr = methodPtr;
		m_source = source;
		m_argTypes = argTypes;
	}
	virtual ~TypedMethod(){	}

	virtual VariantData* invokeOnSource( const uint32& argCount, VariantData** arguments ){
		
		return invoke( argCount, arguments, m_source );		
	}

	virtual Object* getSource(){
		return m_source;
	}

	virtual void setSource( Object* source ){
		m_source = dynamic_cast<SOURCE_TYPE*>( source );
	}
protected:
	SOURCE_TYPE* m_source;
	METHOD_TYPE m_methodPtr;
};


/**
*Base template class for methodsthat DO return values
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodReturn : public TypedMethod<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodReturn( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
		TypedMethod<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ) {
		m_hasReturnValue = true;
	}
	virtual ~TypedMethodReturn(){}
	
protected:
	VariantData m_returnVal;
};


/**
*Method template class for methods have 0 arguments
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg0 : public TypedMethod<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg0( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethod<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 0;
	}

	virtual ~TypedMethodArg0(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 0 == argCount ) {
				(m_source->*m_methodPtr)();
			}
		}

		return NULL;
	}
};

/**
*Method template class for methods have 1 argument
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg1 : public TypedMethod<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg1( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethod<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 1;
	}

	virtual ~TypedMethodArg1(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 1 == argCount ) {
				(m_source->*m_methodPtr)(*(arguments[0]));
			}
		}

		return NULL;
	}
};


/**
*Accepts methds with 2 arguments - no return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg2 : public TypedMethod<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg2( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethod<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 2;
	}

	virtual ~TypedMethodArg2(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 2 == argCount ) {
				(m_source->*m_methodPtr)(	*(arguments[0]),
											*(arguments[1])		);
			}
		}

		return NULL;
	}
};


/**
*Accepts methds with 3 arguments - no return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg3 : public TypedMethod<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg3( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethod<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 3;
	}

	virtual ~TypedMethodArg3(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 3 == argCount ) {
				(m_source->*m_methodPtr)(	*(arguments[0]),
											*(arguments[1]),
											*(arguments[2])		);
			}
		}

		return NULL;
	}
};



/**
*Accepts methds with 4 arguments - no return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg4 : public TypedMethod<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg4( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethod<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 4;
	}

	virtual ~TypedMethodArg4(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 4 == argCount ) {
				(m_source->*m_methodPtr)(	*(arguments[0]),
											*(arguments[1]),
											*(arguments[2]),
											*(arguments[3])		);
			}
		}

		return NULL;
	}
};


/**
*Accepts methds with 5 arguments - no return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg5 : public TypedMethod<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg5( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethod<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 5;
	}

	virtual ~TypedMethodArg5(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 5 == argCount ) {
				(m_source->*m_methodPtr)(	*(arguments[0]),
											*(arguments[1]),
											*(arguments[2]),
											*(arguments[3]),
											*(arguments[4])		);
			}
		}

		return NULL;
	}
};


/**
*Accepts methds with 6 arguments - no return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg6 : public TypedMethod<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg6( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethod<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 6;
	}

	virtual ~TypedMethodArg6(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 6 == argCount ) {
				(m_source->*m_methodPtr)(	*(arguments[0]),
											*(arguments[1]),
											*(arguments[2]),
											*(arguments[3]),
											*(arguments[4]),
											*(arguments[5])		);
			}
		}

		return NULL;
	}
};


/**
*Method template for methods with 1 argument and a return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg0Return : public TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg0Return( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 0;
	}

	virtual ~TypedMethodArg0Return(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );		
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 0 == argCount ) {
				m_returnVal = (m_source->*m_methodPtr)();
			}
		}

		return &m_returnVal;
	}
};


/**
*Method template for methods with 1 argument and a return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg1Return : public TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg1Return( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 1;
	}

	virtual ~TypedMethodArg1Return(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );		
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 1 == argCount ) {
				m_returnVal = (m_source->*m_methodPtr)(*(arguments[0]));
			}
		}

		return &m_returnVal;
	}
};


/**
*Method template for methods with 2 arguments and a return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg2Return : public TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg2Return( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 2;
	}

	virtual ~TypedMethodArg2Return(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );		
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 2 == argCount ) {
				m_returnVal = (m_source->*m_methodPtr)(	*(arguments[0]),
														*(arguments[1]) );
			}
		}

		return &m_returnVal;
	}
};


/**
*Method template for methods with 3 arguments and a return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg3Return : public TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg3Return( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 3;
	}

	virtual ~TypedMethodArg3Return(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );		
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 3 == argCount ) {
				m_returnVal = (m_source->*m_methodPtr)(	*(arguments[0]),
														*(arguments[1]),
														*(arguments[2]) );
			}
		}

		return &m_returnVal;
	}
};


/**
*Method template for methods with 4 arguments and a return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg4Return : public TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg4Return( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 4;
	}

	virtual ~TypedMethodArg4Return(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );		
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 4 == argCount ) {
				m_returnVal = (m_source->*m_methodPtr)(	*(arguments[0]),
														*(arguments[1]),
														*(arguments[2]),
														*(arguments[3]) );
			}
		}

		return &m_returnVal;
	}
};


/**
*Method template for methods with 5 arguments and a return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg5Return : public TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg5Return( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 5;
	}

	virtual ~TypedMethodArg5Return(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );		
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 5 == argCount ) {
				m_returnVal = (m_source->*m_methodPtr)(	*(arguments[0]),
														*(arguments[1]),
														*(arguments[2]),
														*(arguments[3]),
														*(arguments[4]) );
			}
		}

		return &m_returnVal;
	}
};



/**
*Method template for methods with 6 arguments and a return value
*/
template <class SOURCE_TYPE, class METHOD_TYPE> class TypedMethodArg6Return : public TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE> {
public:
	TypedMethodArg6Return( METHOD_TYPE methodPtr, const String& argTypes, SOURCE_TYPE* source=NULL ):
	  TypedMethodReturn<SOURCE_TYPE,METHOD_TYPE>( methodPtr, argTypes, source ){
		m_argCount = 6;
	}

	virtual ~TypedMethodArg6Return(){	}

	virtual VariantData* invoke( const uint32& argCount, 
		                         VariantData** arguments, 
								 Object* source ){

		m_source = dynamic_cast<SOURCE_TYPE*>( source );		
		
		if ( (NULL != m_source) && (NULL != m_methodPtr) ){
			if  ( 6 == argCount ) {
				m_returnVal = (m_source->*m_methodPtr)(	*(arguments[0]),
														*(arguments[1]),
														*(arguments[2]),
														*(arguments[3]),
														*(arguments[4]),
														*(arguments[5]) );
			}
		}

		return &m_returnVal;
	}
};


}; //end of namespace VCF

#endif //_METHOD_H__