/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */

#ifndef LISTENER_H
#define LISTENER_H



/*
*
* Below are 5 macros to simplify the job of adding listener
* support to a class. The 5 macros are:
* 
*    LISTENER_LIST     - declares a listener list (a vector)
*    ADD_LISTENER	   - generates a addListener method
*    REMOVE_LISTENER   - generates a removeListener method
*    HAS_LISTENER      - generates a hasListener method
*    NOTIFIER          - generates a notify method
*
*
* Here's an example class that uses the macros:
*
*
*	class Button {
*	private:
*		LISTENER_LIST(ButtonListener);
*
*	public:
*		ADD_LISTENER(ButtonListener);
*		REMOVE_LISTENER(ButtonListener);
*   
*		NOTIFIER(ButtonListener, handlePress, ButtonEvent);
*		NOTIFIER(ButtonListener, handleClick, ButtonEvent);
*	};
*
* The class that is generated looks as follows:
*
*
*	class Adapter {
*	private:
*		std::vector<ButtonListener*> m_ButtonListenerList;
*
*	public:
*		void addListener(ButtonListener *l);
*		void removeListener(ButtonListener *l);
*
*		void notify_handlePress(ButtonEvent *e);
*		void notify_handleClick(ButtonEvent *e);
*	};
*
*
* Notes About Listener Notification
* ---------------------------------
* During listener notication (i.e. within a listener's callback method):
*
* 1. It is legal for the listener to use addListener/removeListener to change the 
* listener list. The notify_* routine handles this properly. 
* 
* 
* 2. It is legal for the listener to *delete* the object that owns the listener
* list. However, in this case, the listener must call the Event.consume()
* method to stop other listeners from being invoked on a dead object.
* 
*/

#include "Event.h"
#include <map>

namespace VCF{

/** @interface */

class FRAMEWORK_API Listener {
public:
	virtual ~Listener(){};
};

typedef void (Object::*OnEventMethod)(Event* event);

/**
*TEmplate class for creating Listener Adapters that
*
*/
template <class LISTENER_TYPE > class FRAMEWORK_API ListenerAdapter {
public:
	typedef void (LISTENER_TYPE::*OnEventMethod)(Event* event);
	ListenerAdapter( Object* source=NULL ){
		m_source = source;
	}

	Object* getSource(){
		return m_source;
	}

	void setSource( Object* source ){
		m_source = source;
	}
		
	void addEventMethod( const String& eventMethodName, OnEventMethod eventMethod ){
		m_eventMethodMap[eventMethodName] = eventMethod;
	}

	void fireEvent( const String& eventMethodName, Event* event ){
		if ( NULL != m_source ){
			std::map<String,OnEventMethod>::iterator found = m_eventMethodMap.find( eventMethodName );
			if ( found != m_eventMethodMap.end() ){
				OnEventMethod eventMethod = found->second;
				if ( NULL != eventMethod ){
					(m_source->*eventMethod)( event );
				}
			}
		}
	}

	Object* m_source;
protected:
	std::map<String,OnEventMethod> m_eventMethodMap;
};

/**
*Macros for adding event firing and listener list implmentation to a class
*/

#define LISTENER_LIST(Listener) \
    std::vector<Listener*> m_##Listener##List; \
    bool m_##Listener##Changed;


#define ADD_LISTENER(Listener) \
    void add##Listener(Listener *__l) { \
		std::vector<VCF::Listener*>::iterator __p; \
		__p = m_##Listener##List.begin(); \
		while (__p != m_##Listener##List.end()) { \
            if ( *__p++ == __l ) { \
                return; \
			} \
		} \
        m_##Listener##List.push_back(__l); \
	} \

#define REMOVE_LISTENER(Listener) \
    void remove##Listener(Listener *__l) { \
		std::vector<Listener*>::iterator __p; \
		__p = m_##Listener##List.begin(); \
		for ( ; __p != m_##Listener##List.end(); __p++) { \
            if (*__p == __l) { \
				m_##Listener##List.erase(__p); \
				m_##Listener##Changed = true; \
				break; \
			} \
		} \
				} \

#define HAS_LISTENER(Listener) \
    bool has##Listener(Listener *__l) { \
		std::vector<Listener*>::iterator __p; \
		__p = m_##Listener##List.begin(); \
		while (__p != m_##Listener##List.end()) { \
            if (*__p++ == __l) { \
				return true; /* already there? */ \
			} \
		} \
        return false; \
	}

#define NOTIFIER(Listener, Msg, Event) \
    void notify_##Msg( Event *__e) { \
    /* make copy of listener list to be safe */ \
	std::vector<Listener*> __listeners = m_##Listener##List; \
	std::vector<Listener*>::iterator __p; \
	__p = __listeners.begin(); \
	__e->m_consumed = false; \
    bool __changed = m_##Listener##Changed; \
	m_##Listener##Changed = false; \
	while (__p != __listeners.end()) { \
            if (__e->m_consumed) break; \
            if (m_##Listener##Changed) { \
	    /* a listener has been removed from list during notification. */\
	    /* must re-examine list to see if this listener is still there */\
                std::vector<Listener*>::iterator __q; \
		__q = m_##Listener##List.begin(); \
		bool __found = false; \
		while (__q != m_##Listener##List.end()) { \
                    if (*__q++ == *__p) { \
                        __found = true; \
			break; \
                    } \
		} \
		if (!__found) continue; \
            } \
            (*__p)->Msg(__e); __p++;\
	} \
	m_##Listener##Changed = __changed; \
	return; \
    } 





};

#endif //LISTENER_H