/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Together */


#pragma warning (disable : 4786) //vector warnings about string too long...

#ifndef GRAPHICSCONTEXT_H
#define GRAPHICSCONTEXT_H

#include "Object.h"
#include "Rect.h"
#include "Point.h"
#include <map>
#include <vector>
#include "Matrix2D.h"
#include "Color.h"

namespace VCF{

class ContextPeer;
class FontState;
class StrokeState;
class FillState;
class Image;
class Fill;
class Path;
class Stroke;
class Font;

/**
*A Graphics Context provides the lowest level graphics interface to the
*native systems 2D drawing operations. Based loosely on PostScript,
*a Context takes a series of of drawing operations, or commands 
*(lineTo, moveTo, etc), and then executes them by either filling or
*strokeing the path(s) that result from the commands. Thus calling 
*ellipse() will not draw anything till the strokePath() or fillPath()
*methods have been called. All path commands should add their information
*to a stack of some sort that is then looped through and executed
*using the appropriate native graphics calls. At each path command
*a test should be made as to whether the buffer should be cleared.
*The buffer should be cleared only after the stroke of fill methods
*have been  called. For example:
*
*<p><pre>
*	ellipse(23,23,45,67) //added to buffer 
*	moveTo(89,100) //add to buffer
*	lineTo(300,40) //add to buffer
*	strokePath()
*	fillPath()
*	rectangle(200,300,400,400)//buffer cleared, then add to buffer
*</pre></p>
*
*@version 1.0
*@author Jim Crafton
*
*/
class GRAPHICSKIT_API GraphicsContext : public Object {
public:
	GraphicsContext();
	/**
	*Creates a new blank graphics context of the specified width and height
	*/
	GraphicsContext( const unsigned long& width, const unsigned long& height );
	
	/**
	*Create a new context based on a context ID. See ContextPeer getContextID()
	*Doing this should caryy over any of the current settings of the context that the contextID represents.
	*/
	GraphicsContext( const long& contextID );

	virtual ~GraphicsContext();

	virtual void init();

	/**
	*Sets the rendering hints for the context. Rendering hints are used to indicate what the best 
	*case performance for the context should be. For example, if anti-aliasing is specified, then the
	*context will try to anti-aliase whenever possible, according to the capabilities of the
	*graphics subsystem that the context represents. Thus if the context was on a machine that only 
	*supported 16 colors, anti-aliasing might be specified, but would be out of the question due to 
	*the lack of available colors.
	*@param hintName - name of the hint
	*@param hintValue - the value of the hint, i.e. on or off for example
	*/
    void setRenderingHint(const String & hintName, const unsigned long & hintValue);

	/**
	*see above, sets multiple rendering hints at once. 
	*@param renderingHints - a map of all the rendering hints being set in this method
	*/
    void setRenderingHints(const std::map<String, unsigned long> &renderingHints );
	
	/**
	*sets the current font 
	*A copy of the Font is made when this is set
	*/
    void setCurrentFont(Font * font);

	/**
	*returns the current Font
	*/
	Font* getCurrentFont();

	/**
	*sets the current fill
	*/
    void setCurrentFill(Fill * fill);

	/**
	*sets the current stroke
	*/
    void setCurrentStroke(Stroke * stroke);

	/**
	*draws a path. See Path for more info. Basically this simple enumerates all the 
	*path points and uses the current stroke and fill to render the shape
	*/
    void draw(Path * path);

	/**
	*draws an image at the x,y, coordinates
	*/
    void drawImage( const double& x, const double& y, Image * image);

	/**
	*Draws a string at the x,y coordinates
	*/
	void drawString( const double& x, const double& y, const String& text );

	/**
	*gets the capabilites of the GraphicsContext
	*/
    void getCapabilities();    
	
	/**
	*returns current transform matrix for this GraphicsContext instance.
	*Before the matrix is returned, a new matrix is created and then multiplied
	*into the final transform matrix. These matrices represent the various trnasform
	*values (i.e. m_theta, m_scaleX, etc)
	*/
	void setCurrentTransform( Matrix2D* transform );

	Matrix2D* getCurrentTransform();

	/**
	*End of high level gaphics functions. The funtions below are lower level 
	*See ContextPeer for more information on the functions below
	*/

    virtual void setCurrentFontState(FontState * state);

	
	virtual void textAt(const double & x, const double & y, const String & text);

    virtual void rectangle(const double & x1, const double & y1, const double & x2, const double & y2);

	virtual void rectangle( Rect* r ){
		if ( NULL != r ) {
			this->rectangle( r->m_left, r->m_top, r->m_right, r->m_bottom );	
		}
	};

    virtual void circle(const double & x, const double & y, const double & radius);

    virtual void ellipse(const double & x1, const double & y1, const double & x2, const double & y2, const double & angle=0.0);

	virtual void setColor( Color* color );

	virtual Color* getColor();

    virtual void setCurrentStrokeState(StrokeState * state);

    virtual void setCurrentFillState(FillState * state);

    virtual void polyline(std::vector<Point*> & pts);

    virtual void curve(const double& x1, const double& y1, const double& x2, const double& y2,
                          const double& x3, const double& y3, const double& x4, const double& y4);

    virtual void lineTo(const double& x, const double& y);

    virtual void moveTo(const double& x, const double& y);
    
    virtual void fillPath();

    virtual void strokePath();
	
	ContextPeer* getPeer();

	virtual void copyContext( const double& x, const double& y, GraphicsContext* context );

	/**
	*Sets the context's origin for drawing. The default is 0,0 in the top, left of the corner of 
	*screen real estate the context represents
	*/
	void setOrigin( const double& x, const double& y );

	Point* getOrigin();	

	/**
	*sets the current rotation value
	*/
	void setRotation( const double& theta );

	void setTranslation( const double transX, const double& transY );

	void setShear( const double& shearX, const double& shearY );

	void setScale( const double& scaleX, const double& scaleY );
protected:
    std::map<String, unsigned long>  m_renderingHints;
    ContextPeer * m_contextPeer;
	Fill* m_currentFill;
	Stroke* m_currentStroke;
	Font* m_currentFont;
	Matrix2D m_transformMatrix;
	double m_theta;
	double m_xScale;
	double m_yScale;
	double m_xTranslate;
	double m_yTranslate;
	double m_xShear;
	double m_yShear;
	Color m_color;

	void compositeMatrices();	
};

};
#endif //GRAPHICSCONTEXT_HONTEXT_H