/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 



#ifndef IMAGELIST_H
#define IMAGELIST_H


#include "Object.h"

#include <vector>
#include "Enumerator.h"
#include "ImageListListener.h"
#include "Color.h"

namespace VCF
{

class Image;

class GraphicsContext;

class GRAPHICSKIT_API ImageList : public Object{
public:

	ImageList();

	/**
	*Creates a new imagelist from a Image and the 
	*appropriate width and height. 
	*/
	ImageList( Image* listOfImages, const unsigned long& imageWidth, const unsigned long& imageHeight );

	virtual ~ImageList();

	void init();

	/**
	*returns the width of each image in the list, <b><i>NOT</i></b>
	*the width of the list itself. All images in the list
	*will have the same width and height.
	*/
	unsigned long getImageWidth();

	/**
	*returns the height of an image in the list.
	*All images in the list will have the same width and height.
	*/
	unsigned long getImageHeight();

	void setImageWidth( const unsigned long& width );

	void setImageHeight( const unsigned long& height );

	void setTransparentColor( Color* color );

	Color* getTransparentColor();

	void addImage( Image* newImage );
	
	void insertImage( const unsigned long & index, Image* newImage );

	void deleteImage( const unsigned long & index );

	void deleteImage( Image* imageToDelete );

	Enumerator<Image*>* getImages();

	Image* getImage( const unsigned long& index );

	virtual void draw( GraphicsContext* context, const unsigned long& index );

	/**
	*This macro declares a std::vector<> of ListModelListener listeners
	*/
	LISTENER_LIST(ImageListListener);

	/**
	*This macro creates a method for adding a listener to the ListModelListener's ModelValidationListener events
	*/
	ADD_LISTENER(ImageListListener);

	/**
	*This macro creates a method for removing a listener to the ListModelListener's ModelValidationListener events
	*/
	REMOVE_LISTENER(ImageListListener);

	NOTIFIER(ImageListListener, onImageAdded, ImageListEvent);

	NOTIFIER(ImageListListener, onImageDeleted, ImageListEvent);

	NOTIFIER(ImageListListener, onImageListCreated, ImageListEvent);

	NOTIFIER(ImageListListener, onImageListDestroyed, ImageListEvent);

	NOTIFIER(ImageListListener, onImageListSizeChanged, ImageListEvent);
private:
	unsigned long m_imageWidth;
	unsigned long m_imageHeight;
    Color m_transparentColor;
	EnumeratorContainer<std::vector<Image*>, Image*> m_imageContainer;
	std::vector<Image*> m_images;

};

};

#endif //IMAGELIST_H