/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#ifndef _RENDERPATHS_H__
#define _RENDERPATHS_H__

#include "VCF.h"
#include <vector>

namespace VCF {

class Rect;

enum PathCodeType{
  PCT_MOVETO=0,
  PCT_MOVETO_OPEN,
  PCT_CURVETO,
  PCT_LINETO,
  PCT_END
};


class GRAPHICSKIT_API BezierPath {
public:
	BezierPath( const PathCodeType& pathcode, const double& x1=0.0, const double& y1=0.0, 
		         const double& x2=0.0, const double& y2=0.0,
				 const double& x3=0.0, const double& y3=0.0 );

	virtual ~BezierPath();

	virtual void affineTransform( const double matrix[6] );
	
	PathCodeType getPathCode(){
		return m_code;
	}

	void setPathCode( const PathCodeType& pathCode ){
		m_code = pathCode;	
	}

	double getX1(){
		return m_x1;
	}

	double getY1(){
		return m_y1;
	}

	double getX2(){
		return m_x2;
	}

	double getY2(){
		return m_y2;
	}

	double getX3(){
		return m_x3;
	}

	double getY3(){
		return m_y3;
	}

	void setX1( const double& val ){
		m_x1 = val;
	}

	void setY1( const double& val ){
		m_y1 = val;
	}

	void setX2( const double& val ){
		m_x2 = val;
	}

	void setY2( const double& val ){
		m_y2= val;
	}

	void setX3( const double& val ){
		m_x3 = val;
	}

	void setY3( const double& val ){
		m_y3 = val;
	}

	void setAll( const double& x1, const double& y1, 
		         const double& x2, const double& y2,
				 const double& x3, const double& y3 ){
		m_x1 = x1;
		m_y1 = y1;
		m_x2 = x2;
		m_y2 = y2;
		m_x3 = x3;
		m_y3 = y3;

	}

private:
	PathCodeType m_code;
	double m_x1;
	double m_y1;
	double m_x2;
	double m_y2;
	double m_x3;
	double m_y3;
};


class GRAPHICSKIT_API VectorPathPoint {
public:
	VectorPathPoint( const PathCodeType& code=PCT_MOVETO, const double& x1=0.0, const double& y1=0.0 ){
		m_code = code;
		m_x = x1;
		m_y = y1;
	}
	
	virtual ~VectorPathPoint(){}

	PathCodeType m_code;
	double m_x;
	double m_y;
};

class GRAPHICSKIT_API VectorPath {
public:
	VectorPath( const uint32& emptyPtCount=0 );

	/**
	*as circle
	*/
	VectorPath( const double& x, const double& y, const double& radius );

	/**
	*as a rect
	*/
	VectorPath( Rect* rect );

	virtual ~VectorPath();

	VectorPathPoint* addPoint( const PathCodeType& code, const double& x1, const double& y1 );

	void addPoint( VectorPathPoint* pt );

	VectorPath* perturb();

	virtual VectorPath* affineTransform( const double matrix[6] );

	VectorPathPoint* operator[](uint32 pos);

	int32 getPointCount(){
		return m_points.size();
	}

	void addEmptyPoints( const uint32& emptyPtCount );

	Rect* getBounds();
private:
	std::vector<VectorPathPoint*> m_points;
	Rect m_bounds;
};


class GRAPHICSKIT_API VectorPathDash {
public:
	VectorPathDash( VectorPath* vPath=NULL, const double& offset=0.0, 
		            const double* dashArray=NULL, const int32& dashArrayCount=0 );
	
	
	virtual ~VectorPathDash();

	int32 getDashCount(){
		return m_dashes.size();
	}

	double getOffset(){
		return m_offset;
	}

	void setOffset( const double& offset ){
		m_offset = offset;
	}

	void addDash( const double dash ){
		m_dashes.push_back( dash );
	}
	
	VectorPath* getPath(){
		return m_path;
	}

	void setPath( VectorPath* path ){
		m_path = path;
	}
	/**
	*Returns the length of the largest subpath within vpath 
	*/
	int32 getMaxSubPath();

	VectorPath* doDash();
private:
	double m_offset;
	std::vector<double> m_dashes;
	VectorPath* m_path;
};

}; //end of namespace VCF

#endif //_RENDERPATHS_H__