/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Context.h: interface for the Win32Context class.

#ifndef _WIN32CONTEXT_H__
#define _WIN32CONTEXT_H__

#include "VCF.h"
#include "Win32Peer.h"
#include "ContextPeer.h"
//#include "ControlPeer.h"
#include "GraphicsContext.h"
#include "Font.h"
#include "FillState.h"
#include "StrokeState.h"
#include "FontState.h"
#include <deque>

namespace VCF
{


#define MAX_WIN32_CTXT_CACHE_COUNT		4

enum ContextState {
	CTXS_NULL=0,	
	CTXS_HANDLE_VALID=1	
};


enum PrimitiveType{
	PRIMITIVE_MOVE=0,
	PRIMITIVE_LINE,
	PRIMITIVE_POLYLINE,
	PRIMITIVE_BEZIER,
	PRIMITIVE_RECT,
	PRIMITIVE_ELLIPSE,
	PRIMITIVE_ARC,
	PRIMITIVE_CLOSE
};


struct PointOperation {
	double x;
	double y;	
	PrimitiveType primitive;
};

struct TextOperation {
	long x;
	long y;
	String text;
};

class GRAPHICSKIT_API Win32Image;
/**
*A Context provides the lowest level graphics interface to the
*native systems 2D drawing operations. Based loosely on PostScript,
*a Context takes a series of of drawing operations, or commands 
*(lineTo, moveTo, etc), and then executes them by either filling or
*strokeing the path(s) that result from the commands. Thus calling 
*ellipse() will not draw anything till the strokePath() or fillPath()
*methods have been called. All path commands should add their information
*to a stack of some sort that is then looped through and executed
*using the appropriate native graphics calls. At each path command
*a test should be made as to whether the buffer should be cleared.
*The buffer should be cleared only after the stroke of fill methods
*have been  called. For example:
*
*<p><pre>
*	ellipse(23,23,45,67) //added to buffer 
*	moveTo(89,100) //add to buffer
*	lineTo(300,40) //add to buffer
*	strokePath()
*	fillPath()
*	rectangle(200,300,400,400)//buffer cleared, then add to buffer
*</pre></p>
*/
class GRAPHICSKIT_API Win32Context  : public ContextPeer, public Object
{
public:
	Win32Context();
	/**
	*Creates a new HDC from scratch
	*/
	Win32Context( const unsigned long& width, const unsigned long& height );	

	Win32Context( const long& contextID );	

	virtual ~Win32Context();

	virtual void setContext( GraphicsContext* context );

	virtual GraphicsContext* getContext();	

	virtual VCF::FontState* getCurrentFontState();

	virtual long getContextID();

	virtual void setContextID( const long& handle );

	virtual void setCurrentFontState(VCF::FontState * state);

	virtual void drawImage( const double & x, const double & y, Image* image );

    virtual void textAt(const double & x, const double & y, const String & text);

    virtual void rectangle(const double & x1, const double & y1, const double & x2, const double & y2);

    virtual void circle(const double & x, const double & y, const double & radius);

    virtual void ellipse(const double & x1, const double & y1, const double & x2, const double & y2, const double & angle=0.0);

    virtual void setCurrentStrokeState( StrokeState * state );

    virtual void setCurrentFillState( FillState * state );

	virtual StrokeState* getCurrentStrokeState();

    virtual FillState* getCurrentFillState();

    virtual void polyline(std::vector<Point*>& pts);

    virtual void curve(const double & x1, const double & y1, const double & x2, const double & y2,
                         const double & x3, const double & y3, const double & x4, const double & y4);

    virtual void lineTo(const double & x, const double & y);

    virtual void moveTo(const double & x, const double & y);

    virtual void fillPath();

    virtual void strokePath();

	virtual void init();

	virtual void setOrigin( const double& x, const double& y );

	virtual Point* getOrigin();

	virtual void copyContext( const double& x, const double& y, ContextPeer* context );

	virtual bool isMemoryContext();

	//utility functions
	void copyToImage( Win32Image* image );
	
	

	bool handleValid();

	ContextState getState();

	void setState( const ContextState& state );
	
protected:

	HDC getDC();

	bool m_pathStarted;
	std::vector<PointOperation*> m_pathOperations;
	std::vector<TextOperation*> m_textOperations;
	Point m_currentMoveTo;
	Point m_oldOrigin;
	Point m_origin;
	bool m_inFillPath;
	HDC m_dc;
	HBITMAP m_memBitmap;
	FillState m_currentFill;
	StrokeState m_currentStroke;
	FontState m_currentFontState;

	ContextState m_state;

	void clearBuffer();
	void clearPathBuffer();
	void clearTextBuffer();
	void testBuffer();

	long translateFillStyle( const FillStyle& fillState );
	long translateStrokeStyle( const StrokeStyle& strokeStyle );
	long translateHatch( const FillStyle& fillState );
	void execPathOperations();

	/**
	*this function needs to be called before every drawing operation
	*It checks to see if we are over the number of cached DC (4?)
	*if we are then is frees a the handle (a call to ReleaseDC )
	*once this is done we are added to the list and our m_dc member is filled
	*in with a call to GetDC
	*/
	virtual void checkHandle(){};
	
	virtual void releaseHandle(){};

	/*
	void addContext( Win32Context* context ); 

	static void removeContext( Win32Context* context );
	*/

	bool m_isMemoryCtx;
	GraphicsContext* m_context;
	static std::deque<Win32Context*> win32ContextList;
	static int win32ContextCacheCount;
};

};

#endif 
