/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Enum.h: 

#ifndef _ENUM_H__
#define _ENUM_H__


#include "Object.h"
#include <vector>
#include <algorithm>


namespace VCF
{

class FRAMEWORK_API Enum : public Object{
public:
	Enum(){};

	virtual ~Enum(){};

	virtual int next() = 0;

	virtual int end() = 0;

	virtual int get() = 0;
	
	virtual int begin() = 0;

	virtual void set( const int& intVal ) = 0;

	operator int (){
		return get();
	};

	/**
	*sets the enum val from a string. The default behaviour 
	*is to assume the string represents an integer (i.e. "1" or "3")
	*if this is not the case then the enum wil be set to 0 value
	*/
	virtual void setFromString( const String& textVal ){
		int i = 0;		
		sscanf( textVal.c_str(), "%d", &i );
		set( i );
	};
};

template <class ENUM_TYPE> class FRAMEWORK_API TypedEnum : public Enum {
public:

	TypedEnum( const ENUM_TYPE& lower, const ENUM_TYPE& upper ){
		m_enum = lower;
		m_lower = (int)lower;
		m_upper = (int)upper;
		m_enumNames.clear();
		m_namesAvailable = false;
	};
	
	TypedEnum( const ENUM_TYPE& lower, const ENUM_TYPE& upper, const unsigned long& enumNameCount, String* enumNames ){
		m_enum = lower;
		m_lower = (int)lower;
		m_upper = (int)upper;
		m_enumNames.clear();
		if ( enumNameCount > 0 ){
			for ( int i=0;i<enumNameCount;i++ ){
				m_enumNames.push_back( enumNames[i] );
			}
		}
		m_namesAvailable = m_enumNames.size() > 0 ? true : false;
	};

	virtual ~TypedEnum() {};

	virtual int next(){
		int i = (int)m_enum;
		i++;		
		if ( i > m_upper ){			
			i = m_lower;
		}
		m_enum = (ENUM_TYPE)i;
		return i;
	};

	virtual int begin(){
		m_enum = (ENUM_TYPE)m_lower;
		return (int)m_enum;
	};

	virtual int end(){
		m_enum = (ENUM_TYPE)m_upper;
		return (int)m_enum;
	};

	virtual int get(){
		return (int)m_enum;
	};
	
	void set( const ENUM_TYPE& val ){
		m_enum = val;
	};

	virtual void set( const int& intVal ){
		m_enum = (ENUM_TYPE)intVal;
	}

	ENUM_TYPE getValAsEnum(){
		return m_enum;
	};

	operator ENUM_TYPE (){
		return getValAsEnum();
	};	

	operator=( const ENUM_TYPE& val ){
		set( val );
	};	
	
	virtual void setFromString( const String& textVal ){
		if ( true == m_namesAvailable ){
			std::vector<String>::iterator it = m_enumNames.begin();
			int i = 0;
			bool found = false;
			while ( it != m_enumNames.end() ){
				if ( (*it) == textVal ){
					found = true;
					break;
				}
				i++;
				it++;
			}
			if ( true == found ){
				set( i );
			}
			else {
				Enum::setFromString( textVal );	
			}
		}
		else {
			Enum::setFromString( textVal );
		}
	};

	virtual String toString(){
		String result = "";
		
		if ( true == m_namesAvailable ){
			result = m_enumNames[get()];
		}
		else {
			char tmp[25];
			memset( tmp, 0, 25 );
			sprintf( tmp, "%d", get() );
			result += tmp;
		}
		return result;
	};

private:
	ENUM_TYPE m_enum;
	int m_lower;
	int m_upper;
	std::vector<String> m_enumNames;
	bool m_namesAvailable;
};

};

#endif 
