/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

/* Generated by Jesse's fingers */


#ifndef THREAD_H
#define THREAD_H


#include "VCF.h"
#include "Object.h"
#include "Runnable.h"
#include "ThreadPeer.h"

namespace VCF
{

// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
/**
*
* The thread object.  Inherit from this to make your class run in its
*	own thread.  See comments in the Public and Protected sections.
*
*/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

class FRAMEWORK_API Thread : public Object {

/**
* Public functions, to be called by the "Creator" thread, i.e. the code that creates an instance
*	of a class that inherits from Thread and sets it running.
*/

public:    
	Thread();
	virtual ~Thread();

	/**
	* Runs the thread just created.  If the thread is already running, calling this has no effect.
	*/
	bool start();

	/**
	* Stop the thread.  This signals the thread to stop, then waits waitMillisec for it to do so.
	* If it does end by then, great (returns true).  Otherwise, if forcekill is false, the function
	* returns false.  If forcekill is true, the thread is killed by the OS and the function returns
	* false.  pThreadExitCode is the thread exit code, which is valid if Stop() returns true.
	*/
	bool stop( int32 waitMillisec, bool forcekill, uint32 * pThreadExitCode );


/**
* Protected functions, to be called by the class that inherits from Thread.  This is the portable
*	"Thread API".  This provides the basic stuff a worker/UI thread needs to behave well.
*
*/
protected:
	/**
	* This is what is called when start() is called.
	*/
	virtual void main() = 0;

	/**
	* Sleep(msec) - go to sleep for at least msec milliseconds.  Call with msec 0 to relinquish just
	*	the current timeslice.
	*/
	void sleep( uint32 msec );

	/**
	*
	* CheckForStop() - return true if we should stop running asap.  If true, caller should do whatever
	*	cleanup and quit.  How often to call this depends on the requirements of the application;
	*	250msec intervals would be a reasonable rate.  One is not obligated to do this; Stop() can be
	*	used to kill a thread, no matter what.  But then no cleanup happens beyond what the OS does.
	*/
	bool checkForStop();


private:
	// JSL note - these unimplemented functions will cause compile-time errors if code tries
	//	to do a copy by value.  Copy by value is not allowed for instances of Thread.
    Thread (const Thread &);
    Thread& operator=(const Thread&);

	void init();

private:
	ThreadPeer* m_threadPeer;
};



// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
/**
*
* The ThreadMutex object.  This provides thread-exclusive access to some
*	shared resource; class instances in multiple threads coordinate writing
*	to a file, say, by acquiring an agreed-upon write-to-the-file mutex,
*	writing to the file, then releasing the mutex.
*
*/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

class FRAMEWORK_API ThreadMutex : public Object {

public:    
	ThreadMutex();
	virtual ~ThreadMutex();

	/**
	* Try to acquire the mutex.  msec is how and how long to wait.  >=0 means block for up to msec
	*	(if msec is 0, the timeslice is not relinquished, if possible - depends on the OS), if
	*	msec is < 0 then wait forever.
	* *pStopNow is set to true if this thread has been told to exit.  Rules: if Acquire() is called
	*	and the thread is to stop, the mutex is tested with no wait (as if msec were 0).  If available,
	*	it is acquired.  If not, it isn't.  Either way the function returns immediately with pStopNow
	*	true.
	*/
	ThreadMutexAcquire acquire( int32 msec, bool* pStopNow );

	/**
	* Release the mutex.  Return false for error (like the mutex wasn't acquired, or is invalid, ...)
	*/
	bool release();


private:
	// JSL note - these unimplemented functions will cause compile-time errors if code tries
	//	to do a copy by value.  Copy by value is not allowed for instances of Thread.
    ThreadMutex (const ThreadMutex &);
    ThreadMutex& operator=(const ThreadMutex&);

//	void init();

private:
	ThreadMutexPeer* m_threadMutexPeer;
};



// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
/**
*
* The ThreadSignal object.  This provides a means for a thread to block
*	(go to sleep) until another thread signals it to wake up and do
*	something.  This is actually a Mutex in disguise.
*
*/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
// \/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

class FRAMEWORK_API ThreadSignal : public Object {

public:    
	ThreadSignal();
	virtual ~ThreadSignal();

	/**
	* Set the signal state (true = signalled, false = unsignalled).  Return false for error (like
	*	the ThreadSignal is invalid).
	*/
	bool setSignalState( bool signalState );

	/**
	* Check/Wait on signal state.  desiredState is the state caller wants, msec is how long to
	*	wait ( <0 means forever, >=0 means that number of msec, and if 0 the timeslice is not
	*	relinquished if possible - depends on the OS).  *pStopNow is set to true if the thread has
	*	been told to exit.  If the thread is to exit, the signal state is tested with no wait
	*	(as though msec is 0).
	*/
	ThreadSignalState waitOnSignalState( bool desiredState, int32 msec, bool* pStopNow );


private:
	// JSL note - these unimplemented functions will cause compile-time errors if code tries
	//	to do a copy by value.  Copy by value is not allowed for instances of Thread.
    ThreadSignal (const ThreadSignal &);
    ThreadSignal& operator=(const ThreadSignal&);

//	void init();

private:
	ThreadSignalPeer* m_threadSignalPeer;
};


};
#endif //THREAD_H
