/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "Rect.h"
#include "RenderPaths.h"
#include <math.h>
#include "VCFMath.h"

#define CIRCLE_STEPS 128
#define PERTURBATION 2e-3

using namespace VCF;

BezierPath::BezierPath( const PathCodeType& pathcode, const double& x1, const double& y1, 
		         const double& x2, const double& y2,
				 const double& x3, const double& y3 )
{

}


BezierPath::~BezierPath()
{

}

void BezierPath::affineTransform( const double matrix[6] )
{

}


VectorPath::VectorPath( const uint32& emptyPtCount )
{	
	addEmptyPoints( emptyPtCount );
}

VectorPath::VectorPath( const double& x, const double& y, const double& radius )
{
	int32 i;
	
	double theta;	
	
	for (i = 0; i < CIRCLE_STEPS + 1; i++) {
		VectorPathPoint* vec = new VectorPathPoint(PCT_MOVETO);

		vec->m_code = i ? PCT_LINETO : PCT_MOVETO;
		theta = (i & (CIRCLE_STEPS - 1)) * (M_PI * 2.0 / CIRCLE_STEPS);
		vec->m_x = x + radius * cos (theta);
		vec->m_y = y - radius * sin (theta);
		
		if ( i == CIRCLE_STEPS ){
			vec->m_code = PCT_END;		
		}
		this->m_points.push_back( vec );
    }
	
	
}

VectorPath::VectorPath( Rect* rect )
{
	VectorPathPoint* vec = new VectorPathPoint(PCT_MOVETO, rect->m_left, rect->m_top );
	this->m_points.push_back( vec );

	vec = new VectorPathPoint(PCT_LINETO, rect->m_right, rect->m_top );
	this->m_points.push_back( vec );

	vec = new VectorPathPoint(PCT_LINETO, rect->m_right, rect->m_bottom );
	this->m_points.push_back( vec );

	vec = new VectorPathPoint(PCT_LINETO, rect->m_left, rect->m_bottom );
	this->m_points.push_back( vec );

	vec = new VectorPathPoint(PCT_END, rect->m_left, rect->m_top );
	this->m_points.push_back( vec );
}

VectorPath::~VectorPath()
{
	std::vector<VectorPathPoint*>::iterator it = m_points.begin();
	while ( it != m_points.end() ){
		VectorPathPoint* pt = *it;
		delete pt;
		pt = NULL;
		it++;
	}
	m_points.clear();
}

VectorPathPoint* VectorPath::addPoint( const PathCodeType& code, const double& x1, const double& y1 )
{
	VectorPathPoint* result = new VectorPathPoint(PCT_MOVETO);
	result->m_code = code;
	result->m_x = x1;
	result->m_y = y1;

	this->m_points.push_back( result );
	return result;
}

void VectorPath::addPoint( VectorPathPoint* pt )
{
	this->m_points.push_back( pt );
}

void VectorPath::addEmptyPoints( const uint32& emptyPtCount )
{
	VectorPathPoint* vec = NULL;
	for (int i=0;i<emptyPtCount;i++){
		vec = new VectorPathPoint();
		this->m_points.push_back( vec );
	}
}

Rect* VectorPath::getBounds()
{
	if ( true == m_points.empty() ){
		m_bounds.setRect( 0.0, 0.0, 0.0, 0.0 );
	}
	else {
		int i = 0;
		double x0 = 0.0;
		double y0 = 0.0;
		double x1 = 0.0;
		double y1 = 0.0;
		
		if (m_points[0]->m_code == PCT_END)	{
			x0 = y0 = x1 = y1 = 0;
		}
		else {
			x0 = x1 = m_points[0]->m_x;
			y0 = y1 = m_points[0]->m_y;
			int size = m_points.size();
			for (i = 1; i<size; i++) {
				if ( m_points[i]->m_code == PCT_END ){
					break;
				}
				else {
					if (m_points[i]->m_x < x0) x0 = m_points[i]->m_x;
					if (m_points[i]->m_x > x1) x1 = m_points[i]->m_x;
					if (m_points[i]->m_y < y0) y0 = m_points[i]->m_y;
					if (m_points[i]->m_y > y1) y1 = m_points[i]->m_y;
				}
			}
		}
		m_bounds.m_left = x0;
		m_bounds.m_top = y0;
		m_bounds.m_right = x1;
		m_bounds.m_bottom = y1;
	}
	return &m_bounds;
}

VectorPathPoint* VectorPath::operator[](uint32 pos)
{
	return m_points[pos];
}

VectorPath* VectorPath::perturb()
{
	int32 i;
	int32 size;
	VectorPath* newPath = NULL;
	double x, y;
	double x_start, y_start;
	int open;
	
	for (i = 0; m_points[i]->m_code != PCT_END; i++) {
		size = i;
	}
	
	newPath = new VectorPath();
	
	x_start = 0;
	y_start = 0;
	open = 0;
	for (i = 0; i < size; i++) {
		VectorPathPoint* pt = new VectorPathPoint();
		pt->m_code = m_points[i]->m_code;
		x = m_points[i]->m_x + (PERTURBATION * rand ()) / RAND_MAX - PERTURBATION * 0.5;
		y = m_points[i]->m_y + (PERTURBATION * rand ()) / RAND_MAX - PERTURBATION * 0.5;
		if (m_points[i]->m_code == PCT_MOVETO)
		{
			x_start = x;
			y_start = y;
			open = 0;
		}
		else if (m_points[i]->m_code == PCT_MOVETO_OPEN)
			open = 1;
		if (!open && (i + 1 == size || m_points[i + 1]->m_code != PCT_LINETO))
		{
			x = x_start;
			y = y_start;
		}
		pt->m_x = x;
		pt->m_y = y;
		
		if ( i == (size-1) ){
			pt->m_code = PCT_END;
		}

		newPath->addPoint( pt );
    }
	
	
	return newPath;
}

VectorPath* VectorPath::affineTransform( const double matrix[6] )
{
	int32 i;
	int32 size;
	VectorPath* newPath = new VectorPath();
	double x, y;
	
	for (i = 0; m_points[i]->m_code != PCT_END; i++) {
		size = i;
	}
	
	newPath = new VectorPath();
	
	
	for (i = 0; i < size; i++) {
		VectorPathPoint* pt = new VectorPathPoint();
		pt->m_code = m_points[i]->m_code;
		x = m_points[i]->m_x;
		y = m_points[i]->m_y;
		pt->m_x = matrix[0] * x + matrix[2] * y + matrix[4];
		pt->m_y = matrix[1] * x + matrix[3] * y + matrix[5];

		if ( i == (size-1) ){
			pt->m_code = PCT_END;
		}

		newPath->addPoint( pt );
    }
	
	
	return newPath;
}



VectorPathDash::VectorPathDash( VectorPath* vPath, const double& offset, 
		            const double* dashArray, const int32& dashArrayCount )
{
	m_path = vPath;
	m_offset = offset;
	m_dashes.clear();
	if ( (NULL != dashArray) && (dashArrayCount > 0) ){
		for (int i=0;i<dashArrayCount;i++){
			this->m_dashes.push_back( dashArray[i] );
		}
	}
}
	
VectorPathDash::~VectorPathDash()
{

}

int32 VectorPathDash::getMaxSubPath()
{
	int32 maxSubPath = 0;
	int i = 0;
	int start = 0;
	int size = this->m_path->getPointCount();
	for (i = 0; i < size; i++) {
		VectorPathPoint* pt = (*m_path)[i];
		if ( pt->m_code == PCT_END ){
			break;
		}
		else if (pt->m_code == PCT_MOVETO || pt->m_code == PCT_MOVETO_OPEN){
			if ( i - start > maxSubPath ){
				maxSubPath = i - start;
			}
			start = i;
		}
    }
	
	if (i - start > maxSubPath){
		maxSubPath = i - start;
	}	
	
	return maxSubPath;
}



VectorPath* VectorPathDash::doDash()
{	
	if ( NULL == m_path ){
		return NULL;
	}

	int32 maxSubPath = 0;
	
	std::vector<double> dists;


	int32 n_result = 0;
	int32 n_result_max = 0;
	int32 start = 0;
	int32 end = 0;
	int i = 0;
	double total_dist = 0.0;
	
	/* state while traversing dasharray - offset is offset of current dash
	value, toggle is 0 for "off" and 1 for "on", and phase is the distance
	in, >= 0, < dash->dash[offset]. */
	int32 offset = 0;
	int32 toggle = 0;
	double phase = 0.0;
	
	/* initial values */
	int offset_init, toggle_init;
	double phase_init;
	
	maxSubPath = this->getMaxSubPath();
	
	for ( int ii=0;ii<maxSubPath;ii++ ){
		dists.push_back(0.0);
	}

	n_result = 0;
	n_result_max = 16;

	VectorPath* result = new VectorPath( n_result_max );//==> art_new (ArtVpath, n_result_max);
	
	/* determine initial values of dash state */
	int32 nDashes = this->getDashCount();

	toggle_init = 1;
	offset_init = 0;
	phase_init = this->m_offset;
	while (phase_init >= this->m_dashes[offset_init])
    {
		toggle_init = !toggle_init;
		phase_init -= this->m_dashes[offset_init];
		offset_init++;
		if (offset_init == nDashes)
			offset_init = 0;
    }
	
	VectorPath& vpath = *m_path;

	for (start = 0; vpath[start]->m_code != PCT_END; start = end)
    {
		for (end = start + 1; vpath[end]->m_code == PCT_LINETO; end++);
		/* subpath is [start..end) */
		total_dist = 0;
		for (i = start; i < end - 1; i++){
			double dx, dy;
			VectorPathPoint* pt1 = vpath[i];
			VectorPathPoint* pt2 = vpath[i + 1];

			dx = pt2->m_x - pt1->m_x;
			dy = pt2->m_y - pt1->m_y;
			dists[i - start] = sqrt (dx * dx + dy * dy);
			total_dist += dists[i - start];
		}
		if (total_dist <= this->m_dashes[offset_init] - phase_init) {
			/* subpath fits entirely within first dash */
			if (toggle_init){
				for (i = start; i < end; i++){
					VectorPathPoint* pt = vpath[i];
					result->addPoint( pt->m_code, pt->m_x, pt->m_y );
				}
			}
		}
		else {
		/* subpath is composed of at least one dash - thus all
			generated pieces are open */
			double dist;
			
			phase = phase_init;
			offset = offset_init;
			toggle = toggle_init;
			dist = 0;
			i = start;
			if (toggle){
				VectorPathPoint* pt = vpath[i];
				result->addPoint( PCT_MOVETO_OPEN, pt->m_x, pt->m_y );
			}
				
			while (i != end - 1) {
				if (dists[i - start] - dist > this->m_dashes[offset] - phase) {
					/* dash boundary is next */
					double a;					
					double x, y;

					VectorPathPoint* pt1 = vpath[i];
					VectorPathPoint* pt2 = vpath[i + 1];
					
					dist += this->m_dashes[offset] - phase;

					a = dist / dists[i - start];

					x = pt1->m_x + a * (pt2->m_x - pt1->m_x);
					y = pt1->m_y + a * (pt2->m_y - pt1->m_y);
					result->addPoint( toggle ? PCT_LINETO : PCT_MOVETO_OPEN, x, y );
						
					/* advance to next dash */
					toggle = !toggle;
					phase = 0;
					offset++;
					if (offset == nDashes)
						offset = 0;
				}
				else {
					/* end of line in vpath is next */
					phase += dists[i - start] - dist;
					i++;
					dist = 0;
					if (toggle) {
						VectorPathPoint* pt = vpath[i];
						result->addPoint( PCT_LINETO, pt->m_x, pt->m_y );
					}
				}
			}
		}
    }
	
	result->addPoint( PCT_END, 0, 0 );
	
	return result;
  }