/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Context.cpp


#include "Win32Context.h"
#include "Win32Image.h"
#include "FontPeer.h"
#include "ImageBits.h"

using namespace VCF;


std::deque<Win32Context*> Win32Context::win32ContextList;
int Win32Context::win32ContextCacheCount = 0;

Win32Context::Win32Context()
{
	this->init();
}

Win32Context::Win32Context( const unsigned long& width, const unsigned long& height )
{
	this->init();
	
	HDC desktopDC = ::GetDC( ::GetDesktopWindow() );

	m_dc = ::CreateCompatibleDC( desktopDC );
	m_memBitmap = ::CreateCompatibleBitmap( desktopDC, width, height );
	::SelectObject( m_dc, m_memBitmap );
	ReleaseDC( ::GetDesktopWindow(), desktopDC );
	m_isMemoryCtx = true;
	if ( NULL == m_memBitmap ){
		//throw exception 
	}
}

Win32Context::Win32Context( const long& contextID )
{
	this->init();
	this->m_dc = (HDC)contextID;
}

/*
Win32Context::Win32Context( ControlPeer* compPeer )
{
	this->init();
	if ( NULL == compPeer ){
		//throw exception 
	}

	//this->m_dc = GetDC( (HWND)compPeer->getHandleID() );	
}	
*/

Win32Context::~Win32Context()
{
	//this->removeContext( this );

	if ( NULL != m_memBitmap ){
		::DeleteObject( this->m_memBitmap );
		::DeleteDC( m_dc );
	}
	clearBuffer();
}

void Win32Context::init()
{
	m_state = CTXS_NULL;
	this->m_dc = NULL;
	m_context = NULL;
	m_memBitmap = NULL;
	this->m_pathOperations.clear();
	this->m_textOperations.clear();
	this->m_pathStarted = false;
	m_isMemoryCtx = false;
	m_oldOrigin.m_x = 0.0;
	m_oldOrigin.m_y = 0.0;
	m_origin.m_x = 0.0;
	m_origin.m_y = 0.0;
}

void Win32Context::setCurrentFontState(VCF::FontState * state)
{
	this->m_currentFontState = *state;
}

VCF::FontState* Win32Context::getCurrentFontState()
{
	return &this->m_currentFontState;
}

void Win32Context::drawImage( const double & x, const double & y, Image* image )
{
	this->checkHandle();
	Win32Image* win32image = dynamic_cast<Win32Image*>(image);
	if ( NULL != win32image ){
		//HDC bmpDC = win32image->getDC();
		ImageBits* bits = win32image->getImageBits();
		HPALETTE oldPalette = NULL;
		if ( NULL != win32image->m_palette ){
			oldPalette = ::SelectPalette( this->m_dc, win32image->m_palette, FALSE );
			::RealizePalette( this->m_dc );
		}
		SetDIBitsToDevice( m_dc, 
			               x, 
						   y, 
						   win32image->getWidth(), 
						   win32image->getHeight(), 
						   0, 
						   0, 
						   0, 
						   win32image->getHeight(),
			               bits->m_bits,
						   &win32image->m_bmpInfo, 
						   DIB_RGB_COLORS );
		if ( NULL != oldPalette ){
			::SelectPalette( this->m_dc, oldPalette, FALSE );			
		}

	}
	releaseHandle();
}

void Win32Context::textAt(const double & x, const double & y, const String & text)
{	
	this->checkHandle();
	if ( NULL == m_context ){
		//throw exception !
	}
	int oldBkMode = ::SetBkMode( m_dc, TRANSPARENT );

	Font* ctxFont = m_context->getCurrentFont();
	if ( NULL == ctxFont ){
		//throw exception
	}
	FontPeer* fontImpl = ctxFont->getFontPeer();
	
	if ( NULL == fontImpl ){
		//throw exception
	}

	LOGFONT* logFont = (LOGFONT*)fontImpl->getFontHandleID();

	
	HFONT font = CreateFontIndirect( logFont );
	
	Color* fontColor = ctxFont->getColor();
	COLORREF oldFontColor = ::GetTextColor( m_dc );
	if ( NULL != fontColor ){
		oldFontColor = ::SetTextColor( m_dc, fontColor->getRGB() );
	}
	else {
		oldFontColor = ::SetTextColor( m_dc, RGB(0,0,0) );
	}

	HFONT oldFont = (HFONT)::SelectObject( m_dc, font );
	
	char* textToDraw = new char[text.size()+1];
	memset( textToDraw, 0, text.size()+1 );
	text.copy( textToDraw, text.size() );
	RECT r = {0};
	r.left = r.right = x;
	r.top = r.bottom = y;
	UINT formatOptions = 0;
	formatOptions = DT_LEFT | DT_TOP | DT_NOCLIP;
	/* Not using for now
	DRAWTEXTPARAMS extraParams = {0};
	extraParams.cbSize = sizeof(DRAWTEXTPARAMS);
	*/
	
	/*
	*determine the size of the text and adjust the rect accordingly.
	*can't use DrawText to figure out the rect because it doesn't 
	*calc the bounds correctly.
	*/
	SIZE textSize = {0};
	GetTextExtentPoint32( m_dc, text.c_str(), text.size(), &textSize );
	r.right = r.left + textSize.cx;
	r.bottom = r.top + textSize.cy;

	DrawTextEx( m_dc, textToDraw, text.size(), &r, formatOptions, NULL ); 
	
	//clean up after ourselves
	delete[] textToDraw;
	::SelectObject( m_dc, oldFont );
	::DeleteObject( font );
	::SetTextColor( m_dc, oldFontColor );
	::SetBkMode( m_dc, oldBkMode );

	releaseHandle();
}

void Win32Context::rectangle(const double & x1, const double & y1, const double & x2, const double & y2)
{
	testBuffer();
	this->m_pathStarted = true;	
	PointOperation* newPointOp = new PointOperation;
	newPointOp->primitive = PRIMITIVE_RECT;
	newPointOp->x = x1;
	newPointOp->y = y1;
	this->m_pathOperations.push_back( newPointOp );
	
	newPointOp = new PointOperation;
	newPointOp->primitive = PRIMITIVE_RECT;
	newPointOp->x = x2;
	newPointOp->y = y2;
	this->m_pathOperations.push_back( newPointOp );
	
}

void Win32Context::circle(const double & x, const double & y, const double & radius)
{	
	ellipse( x - radius, y - radius, x + radius, y + radius );	
}

void Win32Context::ellipse(const double & x1, const double & y1, const double & x2, const double & y2, const double & angle)
{
	testBuffer();
	m_pathStarted = true;	
	//swap out the values to ensure they are normalized since windows is brain dead about this
	double ax1 = x1;
	double ay1 = y1;
	double ax2 = x2;
	double ay2 = y2;
	
	double tmp = x2;
	if ( ax1 > ax2 ) {
		ax2 = ax1;
		ax1 = tmp;
	}
	tmp = ay2;
	if ( ay1 > ay2 ) {
		ay2 = ay1;
		ay1 = tmp;
	}
	PointOperation* newPointOp = new PointOperation;
	newPointOp->primitive = PRIMITIVE_ELLIPSE;		
	newPointOp->x = ax1;
	newPointOp->y = ay1;
	this->m_pathOperations.push_back( newPointOp );
	
	newPointOp = new PointOperation;
	newPointOp->primitive = PRIMITIVE_ELLIPSE;
	newPointOp->x = ax2;
	newPointOp->y = ay2;
	this->m_pathOperations.push_back( newPointOp );
	
}

void Win32Context::setCurrentStrokeState(VCF::StrokeState * state)
{
	this->m_currentStroke = *state;
}

void Win32Context::setCurrentFillState(VCF::FillState * state)
{
	this->m_currentFill = *state;
}

void Win32Context::polyline( std::vector<Point*> & pts)
{
	testBuffer();
	m_pathStarted = true;
	std::vector<Point*>::iterator it = pts.begin();
	PointOperation* newPointOp = NULL;
	while ( it != pts.end() ){			
		newPointOp = new PointOperation;
		newPointOp->primitive = PRIMITIVE_POLYLINE;
		newPointOp->x = (*it)->m_x;
		newPointOp->y = (*it)->m_y;
		this->m_pathOperations.push_back( newPointOp );
		it++;
	}	
}

void Win32Context::curve(const double & x1, const double & y1, const double & x2, const double & y2,
                         const double & x3, const double & y3, const double & x4, const double & y4 )
{
	testBuffer();
	m_pathStarted = true;
	PointOperation* newPointOp = NULL;
	//start point 1
	newPointOp = new PointOperation;
	newPointOp->primitive = PRIMITIVE_BEZIER;
	newPointOp->x = x1;
	newPointOp->y = y1;
	this->m_pathOperations.push_back( newPointOp );

	//start point 1
	newPointOp = new PointOperation;
	newPointOp->primitive = PRIMITIVE_BEZIER;
	newPointOp->x = x2;
	newPointOp->y = y2;
	this->m_pathOperations.push_back( newPointOp );

	//start point 1
	newPointOp = new PointOperation;
	newPointOp->primitive = PRIMITIVE_BEZIER;
	newPointOp->x = x3;
	newPointOp->y = y3;
	this->m_pathOperations.push_back( newPointOp );

	//start point 1
	newPointOp = new PointOperation;
	newPointOp->primitive = PRIMITIVE_BEZIER;
	newPointOp->x = x4;
	newPointOp->y = y4;
	this->m_pathOperations.push_back( newPointOp );
}

void Win32Context::lineTo(const double & x, const double & y)
{
	testBuffer();
	m_pathStarted = true;
	PointOperation* newPointOp = new PointOperation;
	newPointOp->primitive = PRIMITIVE_LINE;
	newPointOp->x = x;
	newPointOp->y = y;
	this->m_pathOperations.push_back( newPointOp );	
}

void Win32Context::moveTo(const double & x, const double & y)
{
	testBuffer();
	m_pathStarted = true;
	PointOperation* newPointOp = new PointOperation;
	newPointOp->primitive = PRIMITIVE_MOVE;
	newPointOp->x = x;
	newPointOp->y = y;
	this->m_pathOperations.push_back( newPointOp );	
}

void Win32Context::testBuffer()
{
	/**
	*means the path has been finished by a call to fill or stroke path
	*/
	if ( false == this->m_pathStarted ){
		this->clearBuffer();
	}
}

void Win32Context::fillPath()
{
	this->checkHandle();

	m_inFillPath = true;
	this->m_pathStarted = false;
	
	if ( NULL != this->m_dc ){
		LOGBRUSH logBrush = {0};
		logBrush.lbColor = this->m_currentFill.m_Color.getRGB();
		logBrush.lbStyle = this->translateFillStyle( m_currentFill.m_style );
		logBrush.lbHatch = this->translateHatch( m_currentFill.m_style );
		
		HBRUSH fill = ::CreateBrushIndirect( &logBrush );
		HBRUSH oldBrush = (HBRUSH)SelectObject( m_dc, fill );
		
		HPEN nullPen = ::CreatePen( PS_NULL, 0, 0 );
		HPEN oldPen = (HPEN)SelectObject( m_dc, nullPen );			
		
		execPathOperations();
		
		SelectObject( m_dc, oldBrush );
		DeleteObject( fill );
		
		SelectObject( m_dc, oldPen );
		DeleteObject( nullPen );
	}	

	releaseHandle();
}

void Win32Context::strokePath()
{
	this->checkHandle();

	m_inFillPath = false;
	this->m_pathStarted = false;
	
	LOGPEN logPen = {0};
	logPen.lopnColor = this->m_currentStroke.m_Color.getRGB();
	logPen.lopnStyle = this->translateStrokeStyle( this->m_currentStroke.m_style );
	logPen.lopnWidth.x = this->m_currentStroke.m_width;
	
	HPEN stroke = ::CreatePenIndirect( &logPen );
	HPEN oldPen = (HPEN)::SelectObject( m_dc, stroke );
	
	LOGBRUSH logBrush = {0};
	logBrush.lbColor = 0;
	logBrush.lbHatch = 0;
	logBrush.lbStyle = BS_HOLLOW;
	HBRUSH nullBrush = ::CreateBrushIndirect( &logBrush );
	HBRUSH oldBrush = (HBRUSH)::SelectObject( m_dc, nullBrush );
	
	execPathOperations();
	
	SelectObject( m_dc, oldBrush );
	DeleteObject( nullBrush );
	
	SelectObject( m_dc, oldPen );
	DeleteObject( stroke );		
	
	releaseHandle();
}

void Win32Context::clearBuffer()
{
	clearPathBuffer();
	this->clearTextBuffer();
}

void Win32Context::clearPathBuffer()
{
	std::vector<PointOperation*>::iterator pts = this->m_pathOperations.begin();
	while ( pts != m_pathOperations.end() ){
		delete (*pts);
		pts++;
	}
	m_pathOperations.clear();
}

void Win32Context::clearTextBuffer()
{

}

long Win32Context::translateFillStyle( const FillStyle& fillState )
{
	long result = BS_HOLLOW;
	switch ( fillState ){
		case FILL_NONE:{
			result = BS_HOLLOW;	
		}
		break;

		case FILL_SOLID:{
			result = BS_SOLID;	
		}
		break;

		case FILL_DIAGONAL:{
			result = BS_HATCHED;	
		}
		break;

		case FILL_CROSS:{
			result = BS_HATCHED;	
		}
		break;

		case FILL_DIAGCROSS:{
			result = BS_HATCHED;	
		}
		break;

		case FILL_HORZ:{
			result = BS_HATCHED;	
		}
		break;
		
		case FILL_VERT:{
			result = BS_HATCHED;	
		}
		break;
	}

	return result;
}

long Win32Context::translateHatch( const FillStyle& fillState )
{
	long result = 0;
	switch ( fillState ){
		case FILL_DIAGONAL:{
			result = HS_BDIAGONAL;	
		}
		break;

		case FILL_CROSS:{
			result = HS_DIAGCROSS;	
		}
		break;

		case FILL_DIAGCROSS:{
			result = BS_SOLID;	
		}
		break;

		case FILL_HORZ:{
			result = HS_HORIZONTAL;	
		}
		break;
		
		case FILL_VERT:{
			result = HS_VERTICAL;	
		}
		break;
	}

	return result;
}

void Win32Context::execPathOperations()
{
	std::vector<PointOperation*>::iterator pts = this->m_pathOperations.begin();
	long remaingOps = m_pathOperations.size();
	while ( (pts != m_pathOperations.end()) && (remaingOps > 0) ){	
		PointOperation* pointOp = *pts;
		if ( NULL != pointOp ){
			switch ( pointOp->primitive ){
				case PRIMITIVE_MOVE:{
					m_currentMoveTo.m_x = pointOp->x;
					m_currentMoveTo.m_y = pointOp->y;
					::MoveToEx( m_dc, pointOp->x, pointOp->y, NULL );
				}
				break;

				case PRIMITIVE_LINE:{
					::LineTo( m_dc, pointOp->x, pointOp->y );
				}
				break;
				
				case PRIMITIVE_POLYLINE:{
					std::vector<PointOperation*> tmpPts;
					PointOperation* tmpPtOp = NULL;
					while ( (pts != m_pathOperations.end()) && (pointOp->primitive == PRIMITIVE_POLYLINE) ) {
						tmpPtOp = new PointOperation;
						*tmpPtOp = *pointOp;
						tmpPts.push_back( tmpPtOp );
						pts++;
						pointOp = *pts;
						remaingOps --;
					}

					POINT* polyPts = new POINT[tmpPts.size()];
					std::vector<PointOperation*>::iterator it = tmpPts.begin();
					int i =0;
					while ( it != tmpPts.end() ){
						polyPts[i].x = (*it)->x;
						polyPts[i].y = (*it)->y;
						delete (*it);
						it++;
						i++;
					}
					if ( this->m_inFillPath ){
						::Polygon( m_dc, polyPts,  tmpPts.size() );
					}
					else{
						::Polyline( m_dc, polyPts,  tmpPts.size() );
					}					
					
					delete[] polyPts;
					tmpPts.clear();
				}
				break;

				case PRIMITIVE_BEZIER:{
					if ( (remaingOps - 4) < 0 ){
						//throw exception !!
					}
					POINT bezPts[4] = {0};
					bezPts[0].x = pointOp->x;	
					bezPts[0].y = pointOp->y;
					pts++;
					pointOp = *pts;
					bezPts[1].x = pointOp->x;	
					bezPts[1].y = pointOp->y;
					pts++;
					pointOp = *pts;
					bezPts[2].x = pointOp->x;	
					bezPts[2].y = pointOp->y;
					pts++;
					pointOp = *pts;
					bezPts[3].x = pointOp->x;	
					bezPts[3].y = pointOp->y;
					if ( this->m_inFillPath ){
						::BeginPath( m_dc );
						::PolyBezier( m_dc, bezPts, 4 );
						::EndPath( m_dc );
						::FillPath( m_dc );
					}
					else {
						::PolyBezier( m_dc, bezPts, 4 );
					}
					remaingOps -= 4;
				}
				break;

				case PRIMITIVE_RECT:{
					if ( (remaingOps - 1) < 0 ){
						//throw exception !!
					}
					POINT pt = {0};					
					pt.x = pointOp->x;	
					pt.y = pointOp->y;
					pts++;
					pointOp = *pts;
					int fixVal = 0;
					if ( true == m_inFillPath ){
						fixVal = 1;
					}
					::Rectangle( m_dc, pt.x, pt.y, pointOp->x + fixVal, pointOp->y + fixVal );
					remaingOps -= 1;
				}
				break;

				case PRIMITIVE_ELLIPSE:{
					if ( (remaingOps - 1) < 0 ){
						//throw exception !!
					}
					POINT pt = {0};					
					pt.x = pointOp->x;	
					pt.y = pointOp->y;
					pts++;
					pointOp = *pts;
					int fixVal = 0;
					if ( true == m_inFillPath ){
						fixVal = 1;
					}

					::Ellipse( m_dc, pt.x, pt.y, pointOp->x + fixVal, pointOp->y + fixVal );
					remaingOps -= 1;
				}
				break;

				case PRIMITIVE_ARC:{
					
				}
				break;

				case PRIMITIVE_CLOSE:{
					::LineTo( m_dc, this->m_currentMoveTo.m_x, this->m_currentMoveTo.m_y );					
				}
				break;
			}			
		}
		pts ++;
		remaingOps --;
	}
}

long Win32Context::translateStrokeStyle( const StrokeStyle& strokeStyle )
{
	return 0;
}

StrokeState* Win32Context::getCurrentStrokeState()
{
	return &this->m_currentStroke;
}

FillState* Win32Context::getCurrentFillState()
{
	return &this->m_currentFill;
}

long Win32Context::getContextID()
{
	return (long)this->m_dc;
}

void Win32Context::setOrigin( const double& x, const double& y )
{
	this->checkHandle();

	POINT pt = {0};
	::SetViewportOrgEx( m_dc, x, y, &pt );
	m_oldOrigin.m_x = pt.x;
	m_oldOrigin.m_y = pt.y;

	releaseHandle();
}

Point* Win32Context::getOrigin()
{
	this->checkHandle();
	POINT pt = {0};
	::GetViewportOrgEx( m_dc, &pt );
	m_origin.m_x = pt.x;
	m_origin.m_y = pt.y;

	releaseHandle();

	return &m_origin;
}
void Win32Context::copyContext( const double& x, const double& y, ContextPeer* context )
{
	if ( NULL != context ){
		this->checkHandle();

		HDC dc = (HDC)context->getContextID();
		HBITMAP currentBitmap = NULL;
		if ( true == context->isMemoryContext() ){
			currentBitmap = (HBITMAP)::GetCurrentObject( dc, OBJ_BITMAP );
		}
		else if ( true == this->isMemoryContext() ){
			currentBitmap = (HBITMAP)::GetCurrentObject( m_dc, OBJ_BITMAP );
		}

		char* bmpBuffer = NULL;
		long bufferSize = ::GetObject( currentBitmap, sizeof(BITMAP), NULL );
		bmpBuffer = new char[bufferSize];
		memset( bmpBuffer, 0, bufferSize );
		::GetObject( currentBitmap, bufferSize, bmpBuffer );
		BITMAP* bitmap = (BITMAP*)bmpBuffer;

		::BitBlt( this->m_dc, x, y, bitmap->bmWidth, abs(bitmap->bmHeight), dc, 0, 0, SRCCOPY );

		delete [] bmpBuffer;
		bmpBuffer = NULL;

		releaseHandle();

	}
	//else throw exception ???
}

bool Win32Context::isMemoryContext()
{
	return m_isMemoryCtx;
}

void Win32Context::copyToImage( Win32Image* image )
{

}

HDC Win32Context::getDC()
{
	return this->m_dc;
}

void Win32Context::setContext( GraphicsContext* context )
{
	m_context = context;
}

GraphicsContext* Win32Context::getContext()
{
	return m_context;
}



bool Win32Context::handleValid()
{
	bool result = false;
	
	if ( m_state & CTXS_HANDLE_VALID != 0 ){
		result = true;
	}

	return result;
}
/*
void Win32Context::addContext( Win32Context* context )
{
	
	Win32Context::win32ContextList.push_back( context );
	Win32Context::win32ContextCacheCount ++;
	StringUtils::trace( "pushing new \n\t" + context->toString() + 
		                "\nwin32ContextCacheCount = " + StringUtils::toString( Win32Context::win32ContextCacheCount ) + "\n" );
	
}

void Win32Context::removeContext( Win32Context* context )
{
	
	Control* control = context->getContext()->getOwningControl();
	if ( (NULL != control) && (false == win32ContextList.empty()) ){
		Win32Context* firstCtx = Win32Context::win32ContextList[0];
		if ( NULL != firstCtx ){			
			firstCtx->setContextID( 0 );
		}
		Win32Context::win32ContextList.pop_front();
		Win32Context::win32ContextCacheCount --;
		ControlPeer* peer  = control->getPeer();
		StringUtils::trace( "removeing DC...\n" );
		if ( NULL != peer ){
			StringUtils::trace( "/////////peer != NULL !!!!!////////////\n" );
			HDC dc = context->getDC();
			HWND wnd = (HWND)peer->getHandleID();		
			if ( ::ReleaseDC( wnd, dc ) ){
				StringUtils::trace( "Released dc " + StringUtils::toString( (int)dc ) + 
					                "for wnd " + StringUtils::toString( (int)wnd ) + "\n" );
			}
		}
		else {
			StringUtils::trace( ">>>>>>>>>>peer == NULL !!!!!<<<<<<<<<<<<<\n" );
		}
		StringUtils::trace( "popping old \n\t" + context->toString() + 
		                "\nwin32ContextCacheCount = " + StringUtils::toString( Win32Context::win32ContextCacheCount ) + "\n" );
	}
	
}
*/

ContextState Win32Context::getState()
{
	return this->m_state;
}

void Win32Context::setState( const ContextState& state )
{
	this->m_state = state;	
}

void Win32Context::setContextID( const long& handle )
{
	m_dc = (HDC)handle;
}