/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Dialog.cpp
#include "Frame.h"
#include "Dialog.h"
#include "Win32Dialog.h"
#include "Win32ToolKit.h"
#include "Window.h"
#include "Application.h"

using namespace VCF;

BOOL CALLBACK enableThreadWnds( HWND hwnd, LPARAM enabled )
{
	BOOL isEnabled = (BOOL)enabled;
	EnableWindow( hwnd, isEnabled );
	return TRUE;
}


Win32Dialog::Win32Dialog():
	AbstractWin32Component( NULL )
{

}


Win32Dialog::Win32Dialog( Frame* owner, Dialog* component ):
	AbstractWin32Component( component )
{	
	m_owner = owner;		
	m_dialogComponent = component;
	String className = this->getClassName();
	if ( true != this->isRegistered() ){
		registerWin32Class( className, m_wndProc  );
	}
	this->init();

	HWND parent = NULL;
	if ( NULL == m_owner ){
		Application* app = Application::getRunningInstance();
		if ( NULL != app ){
			Window* mainWnd = app->getMainWindow();
			if ( NULL != mainWnd ){
				parent = (HWND)mainWnd->getPeer()->getHandleID();
			}
		}
		else {
			parent = ::GetDesktopWindow();
		}
	}
	else {
		parent = (HWND)this->m_owner->getPeer()->getHandleID();
	}
	String caption = this->m_dialogComponent->getCaption();
	this->m_hwnd = ::CreateWindowEx( this->m_exStyleMask, 
		                             className.c_str(), 
									 caption.c_str(),	
									 this->m_styleMask, 
		                             this->m_bounds.m_left, 
									 this->m_bounds.m_top, 
									 this->m_bounds.getWidth(), 
									 this->m_bounds.getHeight(), 
									 parent, 
									 NULL, 
									 ::GetModuleHandle(NULL), 
									 NULL );

	if ( NULL != this->m_hwnd ){
		Win32Object::registerWin32Object( this );
		::ShowWindow( this->m_hwnd, SW_HIDE );
	}
	this->setCreated( true );
}


Win32Dialog::~Win32Dialog()
{

}

void Win32Dialog::init()
{	
	this->createParams();
	this->m_bounds.setRect( 20, 20, 220, 220 );
}

void Win32Dialog::createParams()
{		
	this->m_styleMask = WS_POPUPWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_DLGFRAME | WS_OVERLAPPED;
	this->m_exStyleMask = WS_EX_RIGHTSCROLLBAR | WS_EX_WINDOWEDGE | WS_EX_DLGMODALFRAME;
}

Rect* Win32Dialog::getClientBounds()
{
	RECT r;
	::GetWindowRect( this->m_hwnd, &r );
	Rect tmp(r.left, r.top, r.right, r.bottom );
	m_clientBounds.setRect(0.0, 0.0, 
		                   tmp.getWidth() - (::GetSystemMetrics( SM_CXSIZEFRAME )*2), 
		                   tmp.getHeight() - 
						         (::GetSystemMetrics( SM_CYCAPTION ) +  
								 (::GetSystemMetrics( SM_CYSIZEFRAME )*2)) );
	return &this->m_clientBounds;
}
void  Win32Dialog::setClientBounds( Rect* bounds )
{

}

LRESULT Win32Dialog::handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc )
{
	LRESULT result = 0;

	switch ( message ) {		

		case WM_CLOSE:{
			//VCF::WindowEvent event( this->getComponent(), WINDOW_EVENT_CLOSE );

			//VCF::Window* window = (VCF::Window*)this->getComponent();
			//window->notify_onWindowClose( &event );
			result = AbstractWin32Component::handleEventMessages( message, wParam, lParam );
		}
		break;

		case WM_DESTROY:{
			PostQuitMessage(0);
		}
		break;

		default: result = AbstractWin32Component::handleEventMessages( message, wParam, lParam );
	}
	return result;
}


ModalReturnType Win32Dialog::showModal()
{
	ModalReturnType result = MRT_NONE;		

	EnumThreadWindows( GetCurrentThreadId(), enableThreadWnds, (LPARAM)FALSE );

	if ( NULL != this->m_hwnd ){		
		EnableWindow( this->m_hwnd, TRUE );
		::ShowWindow( this->m_hwnd, SW_SHOWNORMAL );
		::UpdateWindow( this->m_hwnd );		
		
		SetForegroundWindow( m_hwnd );
		MSG msg = {0};
		bool done = false;
		while (!done) {
			if ( TRUE == PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ){				
				TranslateMessage( &msg );
				DispatchMessage( &msg );						
			}
			else{
				WaitMessage();				
			}
			switch ( msg.message ){
				
				case WM_SYSCOMMAND: {
					if ( SC_CLOSE == msg.wParam ){
						result = MRT_CANCEL;
					}
				}
				break;

				case WM_COMMAND: {
					UINT wID = HIWORD(msg.wParam);
					switch ( wID ){
						case IDCANCEL: {
							result = MRT_CANCEL;
						}
						break;

						case IDOK: {
							result = MRT_OK;
						}
						break;

						case IDYES: {
							result = MRT_YES;
						}
						break;

						case IDNO: {
							result = MRT_NO;
						}
						break;
					}
					
					if ( (wID == IDCANCEL) || (wID == IDOK) || (wID == IDYES) || (wID == IDNO) ){
						PostMessage( msg.hwnd, WM_CLOSE, 0, 0 );
					}
				}
				break;

				case WM_QUIT:{
					done = true;	
				}
				break;
			}
		}		
		DestroyWindow( m_hwnd );
	}
	else {
		//throw exception
	}
	
	EnumThreadWindows( GetCurrentThreadId(), enableThreadWnds, (LPARAM)TRUE );

	HWND parent = NULL;
	
	if ( NULL == m_owner ){
		Control* parentControl = this->m_component->getParent();
		if ( NULL != parentControl ){
			parent = (HWND)parentControl->getPeer()->getHandleID();
		}
		else{
			parent = ::GetDesktopWindow();
		}
	}
	else {
		parent = (HWND)this->m_owner->getPeer()->getHandleID();
	}

	//ShowWindow( parent, SW_NORMAL );
	SetForegroundWindow( parent );
	UpdateWindow( parent );

	return result;
}

void Win32Dialog::show()
{
	EnableWindow( this->m_hwnd, TRUE );
	::ShowWindow( this->m_hwnd, SW_SHOWNORMAL );
	::UpdateWindow( this->m_hwnd );
}

void Win32Dialog::setCaption( const String& caption )
{
	SetWindowText( this->m_hwnd, caption.c_str() );
}

void Win32Dialog::showMessage( const String& message, const String& caption )
{
	HWND activeWnd = GetActiveWindow();
	
	String tmp = caption;
	
	if ( tmp == "" ){
		TCHAR modFilename[MAX_PATH];
		memset(modFilename, 0, MAX_PATH );
		GetModuleFileName( GetModuleHandle(NULL), modFilename, MAX_PATH );
		tmp = modFilename;
		tmp = StringUtils::getFileName( tmp );
	}

	MessageBox( activeWnd, message.c_str(), tmp.c_str(), MB_OK );
}

ModalReturnType Win32Dialog::showMessage( const String& message, const String& caption, const MessageStyle& messageStyle )
{
	ModalReturnType result = MRT_NONE;
	
	UINT mbStyle = 0;
	switch ( messageStyle ){
		case MS_DEFAULT: {
			mbStyle = MB_OK;	
		}
		break;

		case MS_ERROR: {
			mbStyle = MB_OK | MB_ICONERROR;	
		}
		break;

		case MS_INFO: {
			mbStyle = MB_OK | MB_ICONINFORMATION;	
		}
		break;

		case MS_WARNING: {
			mbStyle = MB_OK | MB_ICONWARNING;	
		}
		break;
	}

	HWND activeWnd = GetActiveWindow();
	
	String tmp = caption;
	
	if ( tmp == "" ){
		TCHAR modFilename[MAX_PATH];
		memset(modFilename, 0, MAX_PATH );
		GetModuleFileName( GetModuleHandle(NULL), modFilename, MAX_PATH );
		tmp = modFilename;
		tmp = StringUtils::getFileName( tmp );
	}

	int returnVal = MessageBox( activeWnd, message.c_str(), tmp.c_str(), mbStyle );

	switch ( returnVal ){
		case IDABORT:{
			
		}
		break;

		case IDCANCEL:{
			result = MRT_CANCEL;
		}
		break;
		/*
		case IDCONTINUE:{
			
		}
		break;
		*/
		case IDIGNORE:{
			
		}
		break;

		case IDNO:{
			result = MRT_NO;
		}
		break;

		case IDOK:{
			result = MRT_OK;
		}
		break;

		case IDRETRY:{
			
		}
		break;
		/*
		case IDTRYAGAIN:{
			
		}
		break;
		*/
		case IDYES:{
			result = MRT_YES;
		}
		break;
	}

	return result;
}

void Win32Dialog::close()
{
	::SendMessage( this->m_hwnd, WM_CLOSE, 0, 0 );
}

void Win32Dialog::setFrameStyle( const FrameStyleType& frameStyle )
{
	long style = ::GetWindowLong( this->m_hwnd, GWL_STYLE );
	switch ( frameStyle ){
		case FST_SIZEABLE :{
			style |= WS_OVERLAPPEDWINDOW;			
		}
		break;

		case FST_NOBORDER :{
			style &= ~WS_DLGFRAME;
			style &= ~WS_OVERLAPPEDWINDOW;
			style |= WS_THICKFRAME;			
		}
		break;

		case FST_FIXED :{
			style |= WS_OVERLAPPEDWINDOW;
			style &= ~WS_THICKFRAME;
		}
		break;

		case FST_NOBORDER_FIXED :{
			style &= ~WS_OVERLAPPEDWINDOW;			
		}
		break;		
	}

	::SetWindowLong( this->m_hwnd, GWL_STYLE, style );
	
	Rect* r = this->m_component->getBounds();
	::SetWindowPos( this->m_hwnd, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED );

	::UpdateWindow( this->m_hwnd );
	
	m_component->setBounds( r );

}