/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "Win32Font.h"
#include "Win32FontManager.h"


using namespace VCF;

Win32Font::Win32Font( const String& fontName )
{
	m_pointSize = DEFAULT_FONT_PT_SIZE;
	m_fontName = fontName;
	init();
}

Win32Font::Win32Font( const String& fontName, const double& pointSize )
{
	m_pointSize = pointSize;
	m_fontName = fontName;
	init();
}

Win32Font::~Win32Font()
{
	Win32FontManager::removeFont( this );
}

void Win32Font::init()
{
	memset( &m_logFont, 0, sizeof(m_logFont) );	
	
	HFONT defFont = (HFONT)GetStockObject( DEFAULT_GUI_FONT );
	defFont = NULL;
	if ( NULL != defFont ){
		LOGFONT tmpLogFont = {0};
		GetObject( defFont, sizeof(LOGFONT), &tmpLogFont );
		memcpy( &m_logFont, &tmpLogFont, sizeof(m_logFont));
	}
	else {
		
		m_logFont.lfCharSet = ANSI_CHARSET;//DEFAULT_CHARSET might be better ?
		m_logFont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
		m_logFont.lfEscapement = 0;		
		m_logFont.lfItalic = FALSE;
		m_logFont.lfOrientation = 0;
		m_logFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
		m_logFont.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
		m_logFont.lfQuality = DEFAULT_QUALITY;
		m_logFont.lfStrikeOut = FALSE;
		m_logFont.lfUnderline = FALSE;
		m_logFont.lfWeight = FW_NORMAL;
		m_logFont.lfWidth = 0;//let font mapper choose closest match
	}
	HDC dc = GetDC( NULL );//gets screen DC
	int fontHeight = -MulDiv( m_pointSize, GetDeviceCaps( dc, LOGPIXELSY), 72 );
	ReleaseDC( NULL, dc );
	m_logFont.lfHeight = fontHeight;
	m_fontName.copy( m_logFont.lfFaceName, LF_FACESIZE );

	Win32FontManager::initializeFont( this );
}

ulong32 Win32Font::getFontHandleID()
{
	return (ulong32)(&this->m_logFont);
}

bool Win32Font::isTrueType()
{
	HDC dc = GetDC(NULL);
	HFONT tmpFont = CreateFontIndirect( &this->m_logFont );
	HFONT oldFont = (HFONT)SelectObject( dc, tmpFont );
	
	TEXTMETRIC tm = {0};	
	GetTextMetrics( dc, &tm ); 

	bool result = false;
	if ( (tm.tmPitchAndFamily & TMPF_TRUETYPE) != 0 ){
		result = true;
	}

	SelectObject( dc, oldFont );
	DeleteObject( tmpFont );
	ReleaseDC( NULL, dc );

	return result;
}

Color* Win32Font::getColor()
{
	return &m_color;
}

void Win32Font::setColor( Color* color )
{
	m_color.copy( color );
}

double Win32Font::getPointSize()
{
	HDC dc = GetDC( NULL );
	double ppi = (double)GetDeviceCaps( dc, LOGPIXELSY);
	double result = ((double)m_logFont.lfHeight / ppi) * 72.0;
	ReleaseDC( NULL, dc );

	return result;
}

void Win32Font::setPointSize( const double pointSize )
{	
	HDC dc = GetDC( NULL );
	double ppi = (double)GetDeviceCaps( dc, LOGPIXELSY);	
	m_logFont.lfHeight = (pointSize / 72) * ppi;
	ReleaseDC( NULL, dc );
}

double Win32Font::getPixelSize()
{
	return abs(m_logFont.lfHeight);
}

void Win32Font::setPixelSize( const double pixelSize )
{
	m_logFont.lfHeight = pixelSize;
}

void Win32Font::setBold( const bool& bold )
{
	this->m_logFont.lfWeight = bold ? FW_BOLD : FW_NORMAL;
}

bool Win32Font::getBold()
{
	return (m_logFont.lfWeight == FW_NORMAL) ? false : true;
}

bool Win32Font::getItalic()
{
	return (m_logFont.lfItalic == TRUE) ? true : false;
}

void Win32Font::setItalic( const bool& italic )
{
	m_logFont.lfItalic = italic ? TRUE : FALSE;
}

bool Win32Font::getUnderlined()
{
	return (m_logFont.lfUnderline == TRUE) ? true : false;
}

void Win32Font::setUnderlined( const bool& underlined )
{
	m_logFont.lfUnderline = underlined ? TRUE : FALSE;
}

bool Win32Font::getStrikeOut()
{
	return (m_logFont.lfStrikeOut == TRUE) ? true : false;
}

void Win32Font::setStrikeOut( const bool& strikeout )
{
	m_logFont.lfStrikeOut = strikeout ? TRUE : FALSE;
}

double Win32Font::getShear()
{
	return this->m_shear;
}

void Win32Font::setShear(const double& shear )
{
	this->m_shear = shear;
}

double Win32Font::getAngle()
{
	double tmp = (double)m_logFont.lfEscapement;
	return tmp / 10.0; //Win32 lfEscapement is in tenths of degrees i.e. 334.23 is 3342 
}

void Win32Font::setAngle( const double& angle )
{
	m_logFont.lfEscapement = angle * 10;
	m_logFont.lfOrientation = angle * 10;
}

String Win32Font::getName()
{
	String result = "";
	result = m_logFont.lfFaceName;

	return result;
}

void Win32Font::setName( const String& name )
{
	name.copy( m_logFont.lfFaceName, LF_FACESIZE );
}

GlyphCollection* Win32Font::getGlyphCollection( const String& text )
{
	return NULL;
}

bool Win32Font::isEqual( Object* object )
{
	bool result = false;
	Win32Font* fntObj = dynamic_cast<Win32Font*>( object );
	if ( NULL != fntObj ){
		
		if ( (this->m_fontName == fntObj->m_fontName) &&
			 (this->m_color.isEqual( &(fntObj->m_color) )) &&
			 (this->m_pointSize == fntObj->m_pointSize) ) {
			result = true;
		}
	}

	return result;
}