/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32MenuItem.cpp


#include "Win32MenuItem.h"
#include "BasicException.h"
#include "StringUtils.h"
#include "MenuItem.h"
#include "InvalidPointerException.h"
#include "RuntimeException.h"
#include "MenuBar.h"
#include "PopupMenu.h"

#define BASE_MENU_ID	32778
int globalLastMenuID = BASE_MENU_ID;

using namespace VCF;

std::map<uint32, MenuItem*> Win32MenuItem::menuItemMap;


Win32MenuItem::Win32MenuItem( MenuItem* item )
{	
	this->m_menuItem = item;	
	init();
}

Win32MenuItem::Win32MenuItem()
{	
	init();
}

Win32MenuItem::~Win32MenuItem()
{
	std::map<uint32,MenuItem*>::iterator found = menuItemMap.find( this->m_itemId );
	if ( found != menuItemMap.end() ){
		menuItemMap.erase( found );
	}
	globalLastMenuID --;
	if ( NULL != m_itemHandle ){
		::DeleteObject( m_itemHandle );
	}
}

void Win32MenuItem::init() 
{
	this->m_itemHandle = NULL;
	m_itemId = globalLastMenuID;	
	globalLastMenuID ++;
	menuItemMap[m_itemId] = m_menuItem;
}

void Win32MenuItem::insertSimpleMenuItem( MenuItem* child, HMENU menu )
{

	MenuItemPeer* itemImpl = child->getPeer();
	Win32MenuItem* win32ChildImpl = (Win32MenuItem*)itemImpl;
	MENUITEMINFO info = {0};
	info.cbSize = sizeof(MENUITEMINFO);
	info.fMask = MIIM_ID  | MIIM_STATE | MIIM_TYPE;
	info.fState = MFS_ENABLED;
	if ( true == child->isSeparator() ){
		info.fType |= MFT_SEPARATOR;
	}
	else {
		info.fType = MFT_STRING;
	}
	info.wID = win32ChildImpl->m_itemId;
	
	if ( child->hasChildren() ){		
		info.fMask |= MIIM_SUBMENU;
		info.hSubMenu = (HMENU) itemImpl->getMenuID();
	}
	String itemName = child->getCaption();
	
	info.cch = itemName.size();

	char* tmpName = new char[info.cch+1];
	memset( tmpName, 0, info.cch+1 );
	itemName.copy( tmpName, info.cch );

	info.dwTypeData = tmpName;

	InsertMenuItem( menu, info.wID, TRUE, &info );

	delete [] tmpName;
}

void Win32MenuItem::addChild( MenuItem* child )
{
	if ( NULL != child ){
		insertSimpleMenuItem( child, (HMENU)this->getMenuID() );
	}
}

void Win32MenuItem::insertChild( const unsigned long& index, MenuItem* child )
{
	child->setIndex( index );
	insertSimpleMenuItem( child, m_itemHandle );
}
	
void Win32MenuItem::deleteChild( MenuItem* child )
{
	int index = child->getIndex();
	DeleteMenu( (HMENU)this->getMenuID(), index, MF_BYPOSITION );
}

void Win32MenuItem::deleteChild( const unsigned long& index )
{
	DeleteMenu( (HMENU)this->getMenuID(), index, MF_BYPOSITION );
}

void Win32MenuItem::clearChildren()
{
	int count = GetMenuItemCount( m_itemHandle );
	for ( int i=0;i<count;i++ ){
		DeleteMenu( m_itemHandle, i, MF_BYPOSITION );
	}
}

bool Win32MenuItem::isChecked()
{
	if ( true == m_menuItem->isSeparator() ){
		return false;
	}

	int index = this->m_menuItem->getIndex();

	MENUITEMINFO info = {0};
	info.cbSize = sizeof(MENUITEMINFO);
	info.fMask = MIIM_STATE;
	HMENU menuHandle = (HMENU)this->getMenuID();
	if ( NULL != menuHandle ){
		if ( TRUE == GetMenuItemInfo( menuHandle, index, TRUE, &info ) ){
			return ((info.fState & MFS_CHECKED) != 0) ? true : false;
		}
	}
	return false;
}

void Win32MenuItem::setChecked( const bool& checked )
{
	if ( true == m_menuItem->isSeparator() ){
		return;
	}
	int index = this->m_menuItem->getIndex();
	MENUITEMINFO info = {0};
	info.cbSize = sizeof(MENUITEMINFO);
	info.fMask = MIIM_STATE;
	MenuItem* parent = this->getParent();
	if ( NULL != parent ){
		MenuItemPeer* parentPeer = parent->getPeer();
		HMENU menuHandle = (HMENU)parentPeer->getMenuID();
		if ( NULL != menuHandle ){
			if ( TRUE == GetMenuItemInfo( menuHandle, index, TRUE, &info ) ){
				if ( checked ){
					info.fState |= MFS_CHECKED;	
					info.fState &= ~MFS_UNCHECKED;
				}
				else {
					info.fState &= ~MFS_CHECKED;
					info.fState |= MFS_UNCHECKED;
				}
				
				SetMenuItemInfo( menuHandle, index, TRUE, &info );
			}	
		}
	}
}

bool Win32MenuItem::hasParent()
{
	return false;
}

MenuItem* Win32MenuItem::getParent()
{
	return this->m_menuItem->getParent();
}

MenuItem* Win32MenuItem::getChildAt( const unsigned long& index )
{
	return NULL;
}

bool Win32MenuItem::isEnabled()
{
	if ( true == m_menuItem->isSeparator() ){
		return false;
	}
	int index = this->m_menuItem->getIndex();

	MENUITEMINFO info = {0};
	info.cbSize = sizeof(MENUITEMINFO);
	info.fMask = MIIM_STATE;
	HMENU menuHandle = (HMENU)this->getMenuID();
	if ( NULL != menuHandle ){
		if ( TRUE == GetMenuItemInfo( menuHandle, index, TRUE, &info ) ){
			return ((info.fState & MFS_ENABLED) != 0) ? true : false;
		}
	}
	return false;
}

void Win32MenuItem::setEnabled( const bool& enabled )
{
	if ( true == m_menuItem->isSeparator() ){
		return;
	}
	int index = this->m_menuItem->getIndex();
	MENUITEMINFO info = {0};
	info.cbSize = sizeof(MENUITEMINFO);
	info.fMask = MIIM_STATE;
	MenuItem* parent = this->getParent();
	if ( NULL != parent ){
		MenuItemPeer* parentPeer = parent->getPeer();
		HMENU menuHandle = (HMENU)parentPeer->getMenuID();
		if ( NULL != menuHandle ){
			if ( TRUE == GetMenuItemInfo( menuHandle, index, TRUE, &info ) ){
				if ( true == enabled ){
					info.fState |= MFS_ENABLED;		
					info.fState &= ~MFS_DISABLED;
				}
				else {
					info.fState &= ~MFS_ENABLED;
					info.fState |= MFS_DISABLED;

				}
				
				SetMenuItemInfo( menuHandle, index, TRUE, &info );
			}	
		}
	}
}

bool Win32MenuItem::isVisible()
{
	return true;
}

void Win32MenuItem::setVisible( const bool& visible )
{
	int index = this->m_menuItem->getIndex();
	if ( true == visible ){
		
	}
	else {
		//RemoveMenu( this->m_itemHandle, index, MF_BYPOSITION );
	}
}

bool Win32MenuItem::getRadioItem()
{
	return false;
}

void Win32MenuItem::setRadioItem( const bool& value )
{
	if ( true == m_menuItem->isSeparator() ){
		return;
	}
	int index = this->m_menuItem->getIndex();
	MENUITEMINFO info = {0};
	info.cbSize = sizeof(MENUITEMINFO);
	info.fMask = MIIM_TYPE | MIIM_STATE;
	MenuItem* parent = this->getParent();
	if ( NULL != parent ){
		MenuItemPeer* parentPeer = parent->getPeer();
		HMENU menuHandle = (HMENU)parentPeer->getMenuID();
		if ( NULL != menuHandle ){
			if ( TRUE == GetMenuItemInfo( menuHandle, index, TRUE, &info ) ){
				
				info.fType |= MFT_RADIOCHECK;
				if ( true == value ){						
					info.fState |= MFS_CHECKED;	
				}
				else {
					info.fState &= ~MFS_CHECKED;					
				}
				
				SetMenuItemInfo( menuHandle, index, TRUE, &info );
			}	
		}
	}
}

void Win32MenuItem::setMenuItem( MenuItem* item )
{
	m_menuItem = item;
}

void Win32MenuItem::setCaption( const String& caption )
{
	if ( true == m_menuItem->isSeparator() ){
		return;
	}
	int index = this->m_menuItem->getIndex();
	MENUITEMINFO info = {0};
	info.cbSize = sizeof(MENUITEMINFO);
	info.fMask = MIIM_TYPE;
	MenuItem* parent = this->getParent();
	if ( NULL != parent ){
		MenuItemPeer* parentPeer = parent->getPeer();
		HMENU menuHandle = (HMENU)parentPeer->getMenuID();
		if ( NULL != menuHandle ){
			if ( TRUE == GetMenuItemInfo( menuHandle, index, TRUE, &info ) ){
				info.cch = caption.size();
				
				VCFChar* tmpName = new VCFChar[info.cch+1];
				memset( tmpName, 0, info.cch+1 );
				caption.copy( tmpName, info.cch );
				
				info.dwTypeData = tmpName;
				
				SetMenuItemInfo( menuHandle, index, TRUE, &info );	
				
				delete [] tmpName;				
			}	
		}
	}
}

ulong32 Win32MenuItem::getMenuID()
{
	if ( NULL != m_menuItem ){
		//throw exception !!!
		//throw InvalidPointerException();
		Menu* menuOwner = m_menuItem->getMenuOwner();
		if ( NULL == menuOwner ){
			//throw exception !!!!
			//throw InvalidPointerException();
		}
		if ( (NULL == m_itemHandle) && (NULL != menuOwner)  ){			
			PopupMenu* popupOwner = dynamic_cast<PopupMenu*>( menuOwner );
			if ( NULL != popupOwner ){
				m_itemHandle = ::CreatePopupMenu();
			}
			else {
				MenuBar* menuBarOwner = dynamic_cast<MenuBar*>( menuOwner );
				if ( NULL != menuBarOwner ){
					m_itemHandle = ::CreateMenu();
				}
				else {
					throw RuntimeException(MAKE_ERROR_MSG("Unknown or Invalid Menu Item owner"), __LINE__);
				}
			}

			if ( NULL == m_itemHandle  ){
				throw RuntimeException(MAKE_ERROR_MSG("Error allocation Resources for Menu Item Peer"), __LINE__);	
			}
		}		
	}
	return (ulong32)m_itemHandle;
}

MenuItem* Win32MenuItem::getMenuItemFromID( const uint32 id )
{
	MenuItem* result = NULL;
	
	std::map<uint32, MenuItem*>::iterator found = Win32MenuItem::menuItemMap.find( id );
	if ( found != Win32MenuItem::menuItemMap.end() ){
		result = found->second;
	}
	return result;	
}