/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Object.cpp


#include "Win32Object.h"
#include "ClassRegistry.h"
#include "Control.h"
#include <algorithm>

using namespace VCF;

std::vector< std::string > Win32Object::m_registeredWindowClasses;

std::map<HWND, Win32Object*> Win32Object::m_windowMap;


LRESULT CALLBACK Win32Object_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam){

	Win32Object* win32Obj = Win32Object::getWin32ObjectFromHWND( hWnd );

	if ( NULL != win32Obj ){
		return win32Obj->handleEventMessages( message, wParam, lParam );
	}
	else {
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}


Win32Object::Win32Object()
{	
	this->m_wndProc = Win32Object_WndProc;
	this->m_defaultWndProc = ::DefWindowProc;
	this->init();
}

Win32Object::~Win32Object()
{
	if ( NULL != this->m_hwnd ){		
		::DestroyWindow( this->m_hwnd );		
		std::map<HWND, Win32Object*>::iterator found =  Win32Object::m_windowMap.find( this->m_hwnd ); 		
		if ( found != Win32Object::m_windowMap.end() ){
			Win32Object::m_windowMap.erase( found );
		}
	}
}

void Win32Object::addRegisterWin32Class( const String& className )
{
	Win32Object::m_registeredWindowClasses.push_back( className );
}

void Win32Object::registerWin32Class( const String& className, WNDPROC wndProc )
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	wcex.lpfnWndProc	= (WNDPROC)wndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= (HINSTANCE)::GetModuleHandle(NULL);
	wcex.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= className.c_str();
	wcex.hIconSm		= NULL;

	if ( 0 != RegisterClassEx(&wcex) ){
		addRegisterWin32Class( className );
	}
}

bool Win32Object::isRegistered()
{
	bool result = false;

	std::vector< std::string >::iterator it = std::find( Win32Object::m_registeredWindowClasses.begin(),
													     Win32Object::m_registeredWindowClasses.end(),
					                					 this->getClassName() );

	result = ( it != Win32Object::m_registeredWindowClasses.end() );
	return result;
}

void Win32Object::init()
{
	m_created = false;
	this->m_exStyleMask = 0;
	this->m_styleMask = 0;

	m_peerControl = NULL;

	this->createParams();		
}

void Win32Object::createParams()
{
	this->m_exStyleMask = 0;
	this->m_styleMask = SIMPLE_VIEW;
}

LRESULT Win32Object::handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc )
{
	return ::DefWindowProc( this->m_hwnd, message, wParam, lParam);	
}

Win32Object* Win32Object::getWin32ObjectFromHWND( HWND hwnd )
{
	Win32Object* result = NULL;
	std::map<HWND,Win32Object*>::iterator found = 
		Win32Object::m_windowMap.find( hwnd );
	if ( found != Win32Object::m_windowMap.end() ){
		result = found->second;
	}
	return 	result;
}

HWND Win32Object::getHwnd()
{
	return this->m_hwnd;
}

void Win32Object::registerWin32Object( Win32Object* wndObj )
{
	Win32Object::m_windowMap[ wndObj->getHwnd() ] = wndObj;
	::PostMessage( wndObj->getHwnd(), VCF_CONTROL_CREATE, 0, 0 );
}

LRESULT Win32Object::defaultWndProcedure( UINT message, WPARAM wParam, LPARAM lParam )
{
	LRESULT result = 0;
	if ( NULL != m_defaultWndProc ){
		result = CallWindowProc( m_defaultWndProc, m_hwnd, message, wParam, lParam );				
	}
	return result;
}

void Win32Object::setCreated( const bool& creationComplete )
{
	m_created = creationComplete;
}

bool Win32Object::isCreated()
{
	return m_created;
}

void Win32Object::setPeerControl( Control* control )
{
	this->m_peerControl = control;	
}

Control* Win32Object::getPeerControl()
{
	return m_peerControl;
}