/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

// Win32Window.cpp

#include "Frame.h"
#include "Win32Window.h"
#include "Win32ToolKit.h"
#include "Application.h"


using namespace VCF;

Win32Window::Win32Window( Control* component ):
	AbstractWin32Component( component )
{
	m_internalClose = false;
	this->createParams();

	if ( true != this->isRegistered() ){
		registerWin32Class( this->getClassName(), m_wndProc  );
	}

	Win32ToolKit* toolkit = (Win32ToolKit*)UIToolkit::getDefaultUIToolkit();
	HWND parent = NULL;	

	
	String className = this->getClassName();
	Application* app = Application::getRunningInstance();
	if ( NULL != app ){
		Window* mainWnd = app->getMainWindow();
		if ( NULL != mainWnd ){
			parent = (HWND)mainWnd->getPeer()->getHandleID();
		}
	}
	this->m_hwnd = ::CreateWindowEx( this->m_exStyleMask, 
		                             className.c_str(), 
									 this->m_windowCaption.c_str(),	
									 this->m_styleMask, 
		                             0, 
									 0, 
									 0, 
									 0, 
									 parent, 
									 NULL, ::GetModuleHandle(NULL), NULL );

	if ( NULL != this->m_hwnd ){
		Win32Object::registerWin32Object( this );
		::ShowWindow( this->m_hwnd, SW_SHOWNORMAL );
		::UpdateWindow( this->m_hwnd );
	}
	else {
		//throw exception
	}

	this->setCreated( true );
}

Win32Window::~Win32Window()
{
	
}

void Win32Window::createParams()
{
	this->m_styleMask = WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN;
	m_styleMask &= ~WS_VISIBLE;
}

Rect* Win32Window::getClientBounds()
{
	RECT r;
	::GetWindowRect( this->m_hwnd, &r );
	Rect tmp(r.left, r.top, r.right, r.bottom );
	m_clientBounds.setRect(0.0, 0.0, 
		                   tmp.getWidth() - (::GetSystemMetrics( SM_CXSIZEFRAME )*2), 
		                   tmp.getHeight() - 
						         (::GetSystemMetrics( SM_CYCAPTION ) +  
								 (::GetSystemMetrics( SM_CYSIZEFRAME )*2)) );
	return &this->m_clientBounds;
}
void  Win32Window::setClientBounds( Rect* bounds )
{

}

LRESULT Win32Window::handleEventMessages( UINT message, WPARAM wParam, LPARAM lParam, WNDPROC defaultWndProc )
{
	LRESULT result = 0;

	switch ( message ) {
		case WM_CLOSE:{
			//check if we need to re notify the listeners of the close event
			if ( false == m_internalClose ){
				VCF::WindowEvent event( this->getControl(), WINDOW_EVENT_CLOSE );
				
				VCF::Window* window = (VCF::Window*)this->getControl();
				window->notify_onWindowClose( &event );
				
				//check if the main window is clossing - if it is
				//then close the app !

				Application* app = Application::getRunningInstance();
				if ( NULL != app ){
					Window* mainWindow = app->getMainWindow();
					if ( mainWindow == this->getControl() ){
						::PostMessage( this->m_hwnd, WM_QUIT, 0, 0 ); 
					}
				}
			}
			result = AbstractWin32Component::handleEventMessages( message, wParam, lParam );			
		}
		break;
		default: result = AbstractWin32Component::handleEventMessages( message, wParam, lParam );
	}
	return result;
}

void Win32Window::close()
{
	if ( false == m_internalClose ){
		m_internalClose = true;
		Application* app = Application::getRunningInstance();
		if ( NULL != app ){
			Window* mainWindow = app->getMainWindow();
			if ( mainWindow == this->getControl() ){
				::PostMessage( this->m_hwnd, WM_QUIT, 0, 0 ); 
			}
		}		
	}
}

void Win32Window::setFrameStyle( const FrameStyleType& frameStyle )
{
	long style = ::GetWindowLong( this->m_hwnd, GWL_STYLE );
	switch ( frameStyle ){
		case FST_SIZEABLE :{
			style |= WS_OVERLAPPEDWINDOW;			
		}
		break;

		case FST_NOBORDER :{
			style &= ~WS_DLGFRAME;
			style &= ~WS_OVERLAPPEDWINDOW;
			style |= WS_THICKFRAME;			
		}
		break;

		case FST_FIXED :{
			style |= WS_OVERLAPPEDWINDOW;
			style &= ~WS_THICKFRAME;
		}
		break;

		case FST_NOBORDER_FIXED :{
			style &= ~WS_OVERLAPPEDWINDOW;			
		}
		break;		
	}

	::SetWindowLong( this->m_hwnd, GWL_STYLE, style );
	
	Rect* r = this->m_component->getBounds();
	::SetWindowPos( this->m_hwnd, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED );

	::UpdateWindow( this->m_hwnd );
	
	m_component->setBounds( r );

}

void Win32Window::setFrameTopmost( const bool& isTopmost ) 
{
	if ( true == isTopmost ){
		::SetWindowPos( this->m_hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE );
	}
	else{
		::SetWindowPos( this->m_hwnd, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER );
	}
}