/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//Application.cpp


#include "Application.h"
#include "ApplicationPeer.h"
#include "Window.h"
#include "ResourceBundle.h"
#include "UIToolkit.h"
#include <exception>
#include "ClassRegistry.h"
#include "ApplicationKit.h"
#include "Frame.h"
#include "Dialog.h"

#include "BasicInputStream.h"
#include "ComponentInputStream.h"
#include "Library.h"

#include "InvalidPointerException.h"
//#include "ResourceException.h"

using namespace VCF;


Application* Application::m_appInstance = NULL;

Application::Application()
{
	initApplicationKit();

	//initialize the toolkit
	this->m_mainWindow = NULL;
	
	
	Application::m_appInstance = this;
	UIToolkit* toolkit = UIToolkit::getDefaultUIToolkit();

	this->m_applicationPeer = toolkit->createApplicationPeer();
	m_applicationPeer->setApplication( this );

	m_mainWinHandler = new WindowHandler( this );

	m_mainWinHandler->m_windowClose = (OnWindowEvent)this->onMainWindowClose;
}	

Application::~Application()
{
	delete m_mainWinHandler;
	delete m_applicationPeer;
	if ( NULL != this->m_mainWindow ){
		delete m_mainWindow;
	}

	std::map<String,Library*>::iterator it = m_VPLMap.begin();
	while ( it != m_VPLMap.end() ){
		Library* lib = it->second;
		delete lib;
		lib = NULL;
		it++;
	}
}

void Application::setCommandLineArgs( const std::vector<String>& appCmdLine )
{
	m_cmdLine = appCmdLine;
}

void Application::appMain( int argc, char** argv )
{	
	std::vector<String> commandLine;
	for ( int i=0;i<argc;i++){
		String tmp = argv[i];
		commandLine.push_back( tmp );
	}
	Application::appMain( commandLine );
}

void Application::appMain( const std::vector<String>& appCmdLine )
{
	//main try...catch 
	try {
		if ( NULL == Application::getRunningInstance() ){
			//throw exception			
		}
		else{//enter the main loop		
			
			Application* runningInstance = Application::getRunningInstance();
			/**
			*initialize the impementer first - we are not going to rely	
			*on someone remembering to call the base class functionality of the Application
			*if the Peer returns false then bomb out and terminate
			*/
			ApplicationPeer* appPeer = runningInstance->getPeer();
			if ( NULL == appPeer ){
				//throw exception - this is very BADDDDD
			}
			if ( true != appPeer->initApp( appCmdLine ) ){
				//ouch - something bad happened - terminate the app
				appPeer->terminateApp();
			}
			else {			
				runningInstance->setCommandLineArgs( appCmdLine );	
				
				if ( true != runningInstance->initRunningApplication() ){ //initialization failed
					runningInstance->terminateRunningApplication();
					appPeer->terminateApp();
				}
				else{				
					runningInstance->run();		
					
					runningInstance->terminateRunningApplication();

					appPeer->terminateApp();
				}
			}
		}
	}

	catch ( BasicException& e  ){
		String errString = e.getMessage().c_str();
		
		Dialog::showMessage( errString, MS_ERROR, "Framework Exception" );	

#ifdef _DEBUG
		throw e;
#endif
	}
	catch (std::exception& e){
		String errString = "STL C++ exception throw.\nError : \"";
		String clasName = typeid(e).name();
		errString += " (Exception of type : " + clasName + ")";
		errString += e.what();
		errString += "\".\nApplication exiting abnormally.";
		
		Dialog::showMessage( errString, MS_ERROR, "Framework Exception" );	

#ifdef _DEBUG
		throw e;
#endif
	}
	catch (...){		
		Dialog::showMessage( "Unknown exception occurred. Application exiting abnormally.", 
			                 MS_ERROR, "Framework Exception" );
#ifdef _DEBUG
		throw ;
#endif
	}

	UIToolkit::closeToolkit();
}

Application* Application::getRunningInstance()
{
	return Application::m_appInstance;
}

Window* Application::getMainWindow()
{
	return this->m_mainWindow;
}

void Application::setMainWindow( Window* mainWindow )
{
	if ( NULL != this->m_mainWindow ){
		this->m_mainWindow->removeWindowListener( m_mainWinHandler ); 
	}
	this->m_mainWindow = mainWindow;
	this->m_mainWindow->addWindowListener( m_mainWinHandler ); 
}

ApplicationPeer* Application::getPeer()
{
	return this->m_applicationPeer;
}

bool Application::initRunningApplication()
{
	return true;
}

void Application::onMainWindowClose( WindowEvent* event )
{
	Control* comp = (Control*)event->getSource();
	
	
}

void Application::terminateRunningApplication()
{
	
}

ResourceBundle* Application::getResourceBundle()
{
	return this->m_applicationPeer->getResourceBundle();
}

void Application::run()
{

	m_applicationPeer->run();
	//finished with events - app can terminate

#ifdef USE_VCF_NEW
	//dump any remaing memory - kindof buggy right now - use with caution !!!
	Object::dump();
#endif
	
}

String Application::getFileName()
{
	return this->m_applicationPeer->getFileName();
}

Frame* Application::createFrame( const String& frameClassName )
{
	Frame* result = NULL;
	
	ResourceBundle* resBundle = this->getResourceBundle();
	if ( NULL != resBundle ){
		String vffString = resBundle->getVFF( frameClassName );
		
		if ( false == vffString.empty() ){
			BasicInputStream bis( vffString );
			ComponentInputStream cis( &bis );
			cis.readControl( (Control**)&result );
		}
		else{
			String errMsg = "VFF Resource \"" + frameClassName + "\" has no data.";
			throw InvalidPointerException( MAKE_ERROR_MSG( errMsg ) ); 
		}
	}
	else{
		String errMsg = "Resource \"" + frameClassName + "\" not found.";
		throw InvalidPointerException( MAKE_ERROR_MSG( errMsg ) ); 
	}
	
	return result;
}

void Application::loadFrame( Frame** frame )
{
	
	String frameClassName = (*frame)->getClassName();

	ResourceBundle* resBundle = this->getResourceBundle();
	if ( NULL != resBundle ){
		String vffString = resBundle->getVFF( frameClassName );
		
		if ( false == vffString.empty() ){
			BasicInputStream bis( vffString );
			ComponentInputStream cis( &bis );
			cis.readControl( (Control**)frame );
		}
		else{
			String errMsg = "VFF Resource \"" + frameClassName + "\" has no data.";
			throw InvalidPointerException( MAKE_ERROR_MSG( errMsg ) ); 
		}
	}
	else{
		String errMsg = "Resource \"" + frameClassName + "\" not found.";
		throw InvalidPointerException( MAKE_ERROR_MSG( errMsg ) ); 
	}
}

void Application::loadVPL( const String& vplFileName )
{
	std::map<String,Library*>::iterator found = m_VPLMap.find( vplFileName );
	if ( found == m_VPLMap.end() ){
		Library* vplLib = new Library();
		try{
			vplLib->load( vplFileName );
			typedef void (*InitFunc)();
			InitFunc initFunc = (InitFunc)vplLib->getFunction("initPackage");
			initFunc();

			m_VPLMap[vplFileName] = vplLib; 	
		}
		catch ( BasicException& ex ){
			Dialog::showMessage( ex.getMessage(), MS_ERROR, "Framework Exception" );
		}		
	}	
}

String Application::getCurrentDirectory()
{
	return m_applicationPeer->getCurrentDirectory();
}

void Application::setCurrentDirectory( const String& currentDirectory )
{
	m_applicationPeer->setCurrentDirectory( currentDirectory );
}