/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

//Class.cpp

#include "Class.h"
#include "Method.h"
#include "Property.h"
#include "ClassRegistry.h"
#include "StringUtils.h"
using namespace VCF;

//int propCount = 0;
//int prop_count = 0;

Class::Class( const String& className, const String& classID, const String& superClass )
{
	m_className = className;
	this->m_classID = classID;
	this->m_superClass = NULL;
	this->m_superClassName = superClass;
	m_propertyContainer.initContainer( m_properties );
	this->m_eventContainer.initContainer( this->m_events );
}

Class::~Class()
{
	StringUtils::trace( "destroying class \"" + this->m_className + "\" {\n" );
	std::map<String,Property*>::iterator props = m_properties.begin();
	while ( props != m_properties.end() ){
		Property* prop = props->second;
		//StringUtils::trace( "\tdestroying property \"" + prop->getName() + "\"\n" );
		delete prop;
		//propCount--;
		props++;
	}
	//StringUtils::trace( "prop count = " + StringUtils::toString(propCount) + "\n}\n" );
	m_properties.clear();

	std::map<String,EventPropertySet*>::iterator events = m_events.begin();
	while ( events != m_events.end() ){
		delete events->second;
		events ++;
	}
	m_events.clear();

	std::map<String,Method*>::iterator methods = m_methods.begin();
	while ( methods != m_methods.end() ){
		Method* method = methods->second;
		delete method;
		method = NULL;
		methods++;
	}
	m_methods.clear();

}

String Class::getClassName()
{
	return this->m_className;
}

bool Class::hasProperty( const String& propertyName )
{
	bool result = false;
    std::map<String,Property*>::iterator found = m_properties.find( propertyName );
	result = (found != m_properties.end());
	return result;
}

Property* Class::getProperty( const String& propertyName )
{
	Property* result = NULL;
	
	std::map<String,Property*>::iterator found = m_properties.find( propertyName );
	if ( found != m_properties.end() ){
		result = found->second;
	}

	return result;
}

void Class::addEventSet( EventPropertySet* eventSet )
{
	if ( NULL != eventSet ){
		String type = eventSet->getListenerClassName();
		if ( false == this->hasEventListener( type ) ){
			m_events[type] = eventSet;
		}
	}
}

void Class::addEvent( const String& eventClassName, const String& eventMethodName, 
					const String& eventListenerClassName, const String& eventAdapterClassName )
{
	std::map<String,EventPropertySet*>::iterator found = m_events.find( eventListenerClassName );
	if ( found != m_events.end() ){
		EventPropertySet* eventSet = found->second;
		eventSet->addEvent( eventMethodName );
	}
	else{
		EventPropertySet* newEventSet = new EventPropertySet( eventClassName, 
																eventListenerClassName, 
																eventAdapterClassName );

		newEventSet->addEvent( eventMethodName );
		addEventSet( newEventSet );
	}
}

bool Class::hasEventListener( const String& eventListenerName )
{
	bool result = false;
	std::map<String,EventPropertySet*>::iterator found = m_events.find( eventListenerName );
	result = (found != m_events.end());
	return result;
}

void Class::addProperty( Property* property )
{	
	if ( NULL != property ){
		String name = property->getName();
		if ( false == this->hasProperty( name ) ){
			m_properties[name] = property;
			//propCount ++;
		}
	}
}

void Class::addMethod( Method* method )
{
	if ( NULL != method ){
		String name = method->getName();
		if ( false == this->hasMethod( name ) ){
			m_methods[name] = method;
		}
	}
}

bool Class::hasMethod( const String& methodName ){
	bool result = false;
    std::map<String,Method*>::iterator found = m_methods.find( methodName );
	result = (found != m_methods.end());
	return result;
}

unsigned long Class::getMethodCount(){
	return m_methods.size();
}

Method* Class::getMethod( const String& methodName ){
	Method* result = NULL;
	
	std::map<String,Method*>::iterator found = m_methods.find( methodName );
	if ( found != m_methods.end() ){
		result = found->second;
	}

	return result;
}

void Class::setSource( Object* source )
{
	std::map<String,Property*>::iterator props = m_properties.begin();
	while ( props != m_properties.end() ){
		props->second->setSource( source );
		props++;
	}

	std::map<String,Method*>::iterator methods = m_methods.begin();
	while ( methods != m_methods.end() ){
		Method* method = methods->second;
		method->setSource( source );
		methods++;
	}
}

unsigned long Class::getPropertyCount()
{
	return m_properties.size();
}

String Class::getID()
{
	return this->m_classID;
}

Class* Class::getSuperClass()
{
	if ( NULL == this->m_superClass ){
		ClassRegistry* registry = ClassRegistry::getClassRegistry();
		if ( NULL != registry ) {
			m_superClass = registry->getClass( this->m_superClassName );
		}
	}
	return this->m_superClass;
}

String Class::getClassNameForProperty( Property* property )
{
	String result = "";

	if ( NULL != property ){
		VariantData* val = property->get();
		if ( NULL != val ){
			switch ( val->type ){
				case PROP_INT: {
					result = CLASS_INTEGER;
				}
				break;

				case PROP_LONG: {
					result = CLASS_LONG;
				}
				break;

				case PROP_SHORT: {
					result = CLASS_SHORT;
				}
				break;

				case PROP_ULONG: {
					result = CLASS_ULONG;
				}
				break;

				case PROP_FLOAT: {
					result = CLASS_FLOAT;
				}
				break;

				case PROP_CHAR: {
					result = CLASS_CHAR;
				}
				break;

				case PROP_DOUBLE: {
					result = CLASS_DOUBLE;
				}
				break;

				case PROP_OBJECT: {
					Object* obj = *val;
					if ( NULL != obj ){
						result = obj->getClassName();
					}					
				}
				break;

				case PROP_BOOL: {
					result = CLASS_BOOL;
				}
				break;

				case PROP_STRING: {
					result = CLASS_STRING;
				}
				break;
				
				case PROP_ENUM: {
					result = CLASS_ENUM;
				}
				break;
			}
		}
	}

	return result;
}