/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 

#include "DefaultMenuItem.h"
#include "MenuItemPeer.h"
#include "OutOfBoundsException.h"
#include "UIToolkit.h"
#include "InvalidPeer.h"
#include "Menu.h"
#include <algorithm>

using namespace VCF;

DefaultMenuItem::DefaultMenuItem()
{
	init();
}

DefaultMenuItem::DefaultMenuItem( const String& caption, MenuItem* parent, Menu* menuOwner )
{
	init();
	this->setCaption( caption );
	if ( NULL != parent ){
		parent->addChild( this );
	}
	this->setMenuOwner( menuOwner );
}

DefaultMenuItem::~DefaultMenuItem()
{
	std::vector<MenuItem*>::iterator it = m_menuItems.begin();
	while ( it != m_menuItems.end() ){
		MenuItem* item = *it;
		delete item;
		item = NULL;
		it ++;
	}
	m_menuItems.clear();
	delete m_Peer;
	m_Peer = NULL;
}

void DefaultMenuItem::init() 
{
	m_parent = NULL;
	m_radioItem = false;
	m_visible = true;
	m_data = NULL;
	m_separator = false;
	m_menuOwner = NULL;
	UIToolkit* toolkit = UIToolkit::getDefaultUIToolkit();
	if ( NULL != toolkit ){
		m_Peer = toolkit->createMenuItemPeer( this );
	}
	else {
		//throw exception
	}
	if ( NULL == m_Peer ){
		//throw exception
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	
	m_Peer->setMenuItem( this );

	m_container.initContainer( this->m_menuItems );
}

bool DefaultMenuItem::containsPoint( Point * pt )
{
	return true;
}

Image* DefaultMenuItem::getImage()
{
	return NULL;
}

unsigned long DefaultMenuItem::getIndex()
{
	return m_index;
}

void DefaultMenuItem::setIndex( const unsigned long& index )
{
	m_index = index;
}

void* DefaultMenuItem::getData()
{
	return m_data;
}

void DefaultMenuItem::setData( void* data )
{
	m_data = data;
}

Model* DefaultMenuItem::getModel()
{
	return NULL;
}

void DefaultMenuItem::setModel( Model* model )
{

}

void DefaultMenuItem::paint( GraphicsContext* context, Rect* paintRect )
{

}
	
bool DefaultMenuItem::isSelected()
{
	return m_selected;
}

void DefaultMenuItem::setSelected( const bool& selected )
{
	m_selected = selected;
}

Enumerator<MenuItem*>* DefaultMenuItem::getChildren()
{
	return m_container.getEnumerator();
}

void DefaultMenuItem::addChild( MenuItem* child )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	
	m_menuItems.push_back( child );
	
	child->setParent( this );

	child->setIndex( m_menuItems.size() - 1 );

	child->setMenuOwner( this->getMenuOwner() );
	
	m_Peer->addChild( child );

	ItemEvent event( this, ITEM_EVENT_ADDED );
	notify_onItemAdded( &event );
}

void DefaultMenuItem::insertChild( const unsigned long& index, MenuItem* child )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}

	m_menuItems.insert( m_menuItems.begin() + index, child );

	child->setIndex( index );

	child->setParent( this );

	child->setMenuOwner( this->getMenuOwner() );

	m_Peer->insertChild( index, child );

	ItemEvent event( this, ITEM_EVENT_ADDED );
	notify_onItemAdded( &event );
}
	
void DefaultMenuItem::deleteChild( MenuItem* child )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}

	m_Peer->deleteChild( child );

	std::vector<MenuItem*>::iterator found = std::find( m_menuItems.begin(), m_menuItems.end(), child );
	if ( found != m_menuItems.end() ){
		ItemEvent event( this, ITEM_EVENT_DELETED );
		notify_onItemDeleted( &event );

		m_menuItems.erase( found );
		delete child;
		child = NULL;
	}
}

void DefaultMenuItem::deleteChild( const unsigned long& index )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}

	m_Peer->deleteChild( index );

	std::vector<MenuItem*>::iterator found = m_menuItems.begin() + index;
	if ( found != m_menuItems.end() ){
		
		ItemEvent event( this, ITEM_EVENT_DELETED );
		notify_onItemDeleted( &event );

		m_menuItems.erase( found );
		delete *found;
	}		
}

void DefaultMenuItem::clearChildren()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	std::vector<MenuItem*>::iterator it = m_menuItems.begin();
	while ( it != m_menuItems.end() ){
		delete *it;
		it++;
	}
	m_menuItems.clear();

	m_Peer->clearChildren();
}

bool DefaultMenuItem::isChecked()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	return m_Peer->isChecked();
}

void DefaultMenuItem::setChecked( const bool& checked )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	m_Peer->setChecked( checked );
}

bool DefaultMenuItem::hasParent()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	return m_Peer->hasParent();
}

MenuItem* DefaultMenuItem::getParent()
{
	return m_parent;
}

void DefaultMenuItem::setParent( MenuItem* parent )
{
	m_parent = parent;
}

MenuItem* DefaultMenuItem::getChildAt( const unsigned long& index )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	
	if ( index >= m_menuItems.size() ) {
		throw OutOfBoundsException(MAKE_ERROR_MSG(OUT_OF_BOUNDS_EXCEPTION), __LINE__);
	}
	return this->m_menuItems[index];
}

bool DefaultMenuItem::isEnabled()
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	return m_Peer->isEnabled();
}

void DefaultMenuItem::setEnabled( const bool& enabled )
{
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	m_Peer->setEnabled( enabled );

	ItemEvent event( this, ITEM_EVENT_CHANGED );
	notify_onItemChanged( &event );
}

bool DefaultMenuItem::isVisible()
{
	return m_visible;
}

void DefaultMenuItem::setVisible( const bool& visible )
{
	m_visible = visible;
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}
	m_Peer->setVisible( visible );

	ItemEvent event( this, ITEM_EVENT_CHANGED );
	notify_onItemChanged( &event );
}

bool DefaultMenuItem::getRadioItem()
{
	return m_radioItem;
}

void DefaultMenuItem::setRadioItem( const bool& value )
{
	m_radioItem = value;
	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}

	m_Peer->setRadioItem( m_radioItem );

	ItemEvent event( this, ITEM_EVENT_CHANGED );
	notify_onItemChanged( &event );
}

void DefaultMenuItem::setCaption( const String& caption )
{
	m_caption = caption;

	if ( NULL == this->m_Peer ){
		throw InvalidPeer(MAKE_ERROR_MSG(NO_PEER), __LINE__);
	}

	m_Peer->setCaption( m_caption );

	ItemEvent event( this, ITEM_EVENT_TEXT_CHANGED );
	notify_onItemChanged( &event );
}

String DefaultMenuItem::getCaption() 
{
	return m_caption;
}

MenuItemPeer* DefaultMenuItem::getPeer()
{
	return m_Peer;
}

bool DefaultMenuItem::hasChildren()
{
	return ! this->m_menuItems.empty();
}

void DefaultMenuItem::click()
{
	MenuItemEvent event( this );
	notify_onMenuItemClicked( &event );
}

void DefaultMenuItem::update()
{
	MenuItemEvent event( this );
	//notify_onMenuItemClicked( &event );
}

Menu* DefaultMenuItem::getMenuOwner()
{
	return m_menuOwner;
}

void DefaultMenuItem::setMenuOwner( Menu* menuOwner )
{
	m_menuOwner = menuOwner;
	std::vector<MenuItem*>::iterator it = m_menuItems.begin();
	while ( it != m_menuItems.end() ){
		MenuItem* child = *it;
		if ( NULL != child ) {
			child->setMenuOwner( menuOwner );
		}
		it++;
	}
}

bool DefaultMenuItem::isSeparator()
{
	return m_separator;
}

void DefaultMenuItem::setSeparator( const bool& separator )
{
	m_separator = separator;
}