/**
*Visual Component Framework for C++
*
*Copyright  2000 Jim Crafton
*
*This program is free software; you can redistribute it and/or
*modify it under the terms of the GNU General Public License
*as published by the Free Software Foundation; either version 2
*of the License, or (at your option) any later version.
*
*This program is distributed in the hope that it will be useful,
*but WITHOUT ANY WARRANTY; without even the implied warranty of
*MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*GNU General Public License for more details.
*
*You should have received a copy of the GNU General Public License
*along with this program; if not, write to the Free Software
*Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*Contact Jim Crafton at ddiego@one.net for more information about the 
*Visual Component Framework
*/ 
//TabbedPages.cpp

#include "TabbedPages.h"
#include "Basic3DBorder.h"
#include "DefaultTabModel.h"
#include "TabModelListener.h"
#include "DefaultTabPage.h"
#include "Panel.h"
#include "FillState.h"

using namespace VCF;

TabbedPages::TabbedPages():
	CustomControl( true )
{
	init();
}

void TabbedPages::init()
{
	this->setBorder( NULL );
	//add a listener to the tab model
	this->setContainer( this );	
	TabModelHandler* tabModelListener = new TabModelHandler( this );

	tabModelListener->m_tabPageAdded = (OnTabEvent)onTabPageAdded;
	tabModelListener->m_tabPageRemoved = (OnTabEvent)onTabPageRemoved;
	tabModelListener->m_tabPageSelected = (OnTabEvent)onTabPageSelected;

	this->addEventHandler( "TabbedPages::tabModelListener", tabModelListener );

	DefaultTabModel* defaultTabModel = new DefaultTabModel();
	this->m_model = defaultTabModel;
	m_model->addTabModelListener( tabModelListener ); 

	m_borderWidth = 8;	
}

TabbedPages::~TabbedPages()
{
	if ( NULL != m_model ){
		delete m_model;
	}
}


void TabbedPages::paint( GraphicsContext* context )
{
	CustomControl::paint( context );
	Rect tabsRect( 0.0, 0.0, 0.0, 0.0 );
	Rect* bounds = this->getBounds();
	FillState fill;
	fill.m_Color = *( this->getColor() );
	context->rectangle( 0, 0, bounds->getWidth(), bounds->getHeight() );
	context->setCurrentFillState( &fill );
	context->fillPath();
	
	if ( NULL != m_model ){
		Enumerator<TabPage*>* pages = m_model->getPages();
		if ( NULL != pages ){
			int currentLeft = m_tabAreaBounds.m_left;
			
			while ( true == pages->hasMoreElements() ){
				TabPage* page = pages->nextElement();
				if ( NULL != page ){
					tabsRect.setRect( currentLeft, 3, 
						              currentLeft+ 50, 3+page->getPreferredHeight() );					
					page->paint( context, &tabsRect );
				}
				currentLeft += 50;
				Control* component = page->getPageComponent();
				
				if ( NULL != component ){
					Rect tmp( *(component->getBounds()) );
					tmp.inflate( 3,3 );
					m_activePageBorder.paint( &tmp, context );					
				}
			}
			
		}
	}
	this->paintChildren( context );
}

TabModel* TabbedPages::getModel()
{
	return m_model;
}

void TabbedPages::setTabModel( TabModel* model )
{
	m_model = model;
}

void TabbedPages::onTabPageAdded( TabModelEvent* event )
{

}

void TabbedPages::onTabPageRemoved( TabModelEvent* event )
{

}

void TabbedPages::onTabPageSelected( TabModelEvent* event )
{
	TabPage* page = event->getTabPage();
	if ( true == page->isSelected() ){
		Enumerator<Control*>* children = this->getChildren();
		while ( true == children->hasMoreElements() ){
			Control* comp = children->nextElement();
			if ( comp != page->getPageComponent() ){
				comp->setVisible( false );
			}
		}
		page->getPageComponent()->setVisible( true );
	}
	this->repaint();
}

TabPage* TabbedPages::addNewPage( const String& caption )
{
	DefaultTabPage* page = new DefaultTabPage();
	page->setModel( this->getModel() );
	page->setPageName( caption );	
	Panel* sheet = new Panel();
	page->setPageComponent( sheet );	
	sheet->setBorder( NULL );	
	this->add( sheet, ALIGN_CLIENT );
	sheet->setVisible( true );
	
	TabModel* model = this->getModel();
	model->addTabPage( page );
	model->setSelectedPage( page );

	return page;
}

void TabbedPages::resizeChildren()
{
	if ( true == alignWork() ){		
		Rect* bounds = this->getBounds();
		m_tabAreaBounds.setRect( 0, 0, bounds->getWidth(), bounds->getHeight() );
		m_tabAreaBounds.inflate( -m_borderWidth, -m_borderWidth );
		m_tabAreaBounds.m_top = m_tabAreaBounds.m_top + 12;

		this->doAlign( ALIGN_TOP, &m_tabAreaBounds );
		this->doAlign( ALIGN_BOTTOM, &m_tabAreaBounds );
		this->doAlign( ALIGN_LEFT, &m_tabAreaBounds );
		this->doAlign( ALIGN_RIGHT, &m_tabAreaBounds );
		this->doAlign( ALIGN_CLIENT, &m_tabAreaBounds );
	}
}

void TabbedPages::mouseDown( MouseEvent* event )
{	
	CustomControl::mouseDown( event );	
	if ( true == this->m_tabAreaBounds.containsPt( event->getPoint() ) ){
		//find tab
		TabModel* model = this->getModel();
		if ( NULL != model ){
			Enumerator<TabPage*>* pages = model->getPages();
			if ( NULL != pages ){
				while ( true == pages->hasMoreElements() ){
					TabPage* page = pages->nextElement();
					if ( NULL != page ){
						if ( true == page->containsPoint( event->getPoint() ) ){
							this->m_model->setSelectedPage( page );
							this->resizeChildren();
							break;
						}
					}
				}
			}
		}
	}
}

void TabbedPages::mouseMove( MouseEvent* event )
{
	CustomControl::mouseMove( event );	
}

void TabbedPages::mouseUp( MouseEvent* event )
{
	CustomControl::mouseUp( event );	
}

ulong32 TabbedPages::getBorderWidth()
{
	return this->m_borderWidth;
}

void TabbedPages::setBorderWidth( const ulong32& borderWidth )
{
	m_borderWidth = borderWidth;
}